import inspect
import re
import sys
import warnings
import time
import weakref

from itertools import count, izip



__all__ = [
    "OrderedSet",
    "RequestLocalDescriptor",
    "lazystring",
    "asbool",
    "LRUCache",
    "disable_runtime_checks",
    ]

# Stolen from PasteDeploy
def asbool(obj):
    if isinstance(obj, (str, unicode)):
        obj = obj.strip().lower()
        if obj in ['true', 'yes', 'on', 'y', 't', '1']:
            return True
        elif obj in ['false', 'no', 'off', 'n', 'f', '0']:
            return False
        else:
            raise ValueError(
                "String is not true/false: %r" % obj)
    return bool(obj)


def assert_bool_attr(name, state, exc):
    """
    Raises 'exc' if given flag is not equal to 'state'.
    """
    def entangle(func):
        def wrapper(self, *args, **kw):
            if getattr(self,name) == state:
                return func(self, *args, **kw)
            raise exc
        wrapper.func_name = func.func_name
        wrapper.__dict__ = func.__dict__.copy()
        return wrapper
    return entangle




def pre_post_wrapper(pre=None, post=None, lock=None):
    """
    Decorates a method excecuting pre and post methods around it.

    Can be used to decorate the same method in different subclasses and takes
    care that pre is executed only once at the first cooperative call and post
    once at the end of the cooperative call-chain.

        >>> entries = 0
        >>> exits = 0
        >>> def pre(*args, **kw): global entries; entries += 1
        >>> def post(*args, **kw): global exits; exits += 1
        >>> class A(object):
        ...     @pre_post_wrapper(pre, post)
        ...     def say_name(self, name):
        ...         print name
        >>> class B(A):
        ...     @pre_post_wrapper(pre, post)
        ...     def say_name(self, name):
        ...         super(B, self).say_name(name)
        ...         print name
        >>> class C(B):
        ...     @pre_post_wrapper(pre, post)
        ...     def say_name(self, name):
        ...         super(C, self).say_name(name)
        ...         print name
        >>> c = C()
        >>> c.say_name('foo')
        foo
        foo
        foo
        >>> entries
        1
        >>> exits
        1

    A reentrant lock can be passed to syncronize the wrapped method. It is a
    must if the instance is shared among several threads.
    """
    def entangle(func):
        def wrapper(self, *args, **kw):
            counter_name = '__%s_wrapper_counter' % func.func_name
            if lock:
                lock.aquire()
            try:
                counter = getattr(self, counter_name, 0) + 1
                setattr(self, counter_name, counter)
                if counter == 1 and pre:
                    pre(self, *args, **kw)
                output = func(self, *args, **kw)
                counter = getattr(self, counter_name)
                setattr(self, counter_name, counter - 1)
                if counter == 1:
                    delattr(self, counter_name)
                    if post: post(self, *args, **kw)
            finally:
                if lock:
                    lock.release()
            return output
        wrapper.func_name = func.func_name
        wrapper.__dict__ = func.__dict__.copy()
        return wrapper
    return entangle

def pre_post_hooks(pre_name=None, post_name=None, lock=None):
    def entangle(func):
        def wrapper(self, *args, **kw):
            counter_name = '__%s_wrapper_counter' % func.func_name
            bases = list(inspect.getmro(self.__class__))
            if lock:
                lock.aquire()
            try:
                counter = getattr(self, counter_name, 0) + 1
                setattr(self, counter_name, counter)
                if counter == 1 and pre_name:
                    for base in bases:
                        try:
                            # make sure we use the hook  defined in base
                            base.__dict__[pre_name](self, *args, **kw)
                        except KeyError:
                            pass
                output = func(self, *args, **kw)
                counter = getattr(self, counter_name)
                setattr(self, counter_name, counter - 1)
                if counter == 1:
                    delattr(self, counter_name)
                    if post_name:
                        for base in bases:
                            try:
                                base.__dict__[post_name](self, *args, **kw)
                            except KeyError:
                                pass
            finally:
                if lock:
                    lock.release()
            return output
        wrapper.func_name = func.func_name
        wrapper.__dict__ = func.__dict__.copy()
        return wrapper
    return entangle

class CachedInspect(object):
    """
    In general "inspect" calls are extremely expensive.  However, if we cache them
    on an object by object basis, we should be able to speed things up a 
    """
    def __init__(self):
        self.functions = weakref.WeakKeyDictionary()
        self.methods = weakref.WeakKeyDictionary()
        self.argspec = weakref.WeakKeyDictionary()
    
    def is_function(self, obj):
        truth = self.functions.get(obj, None)
        if truth is None:
            truth = self.functions[obj] = inspect.isfunction(obj)
        return truth

    def is_method(self, obj):
        truth = self.methods.get(obj, None)
        if truth is None:
            truth = self.methods[obj] = inspect.ismethod(obj)
        return truth
    
    def getargspec(self, obj):
        spec = self.argspec.get(obj, None)
        if spec is None:
            spec = self.argspec[obj] = inspect.getargspec(obj)
        return spec
callable_cache = CachedInspect()

def callable_wo_args(obj):
    if callable(obj) and (callable_cache.is_function(obj) or callable_cache.is_method(obj)):
        argspec = callable_cache.getargspec(obj)
        args = argspec[0]
        defaults = argspec[3]
        arg_length = callable_cache.is_method(obj) and 1 or 0
        return (
            (args is not None and (len(args) == arg_length)) or
            (defaults is not None and len(args) == len(defaults))
            )
    return False



_id_RE = re.compile(r'(\w+)+(?:-(\d))*')
def unflatten_args(child_args):
    new = {}
    for k,v in child_args.iteritems():
        splitted = k.split('.',1)
        id = splitted[0]
        if len(splitted) == 2:
            rest = splitted[1]
            if not id:
                new[rest] = v
                continue
            for_id = new.setdefault(id, {})
            for_id.setdefault('child_args', {})[rest] = v
        else:
            for_id = new.setdefault(id,{})
            for key, val  in v.iteritems():
                for_id.setdefault(key,val)

    convert = set()
    for k,v in new.items():
        m = _id_RE.match(k)
        if not m:
            raise ValueError(
                "%r is not a valid id to reference a child" % k
                )
        id, n = m.groups()
        if n is not None:
            new.pop(k,None)
            convert.add(id)
            for_id = new.setdefault(id, {})
            for_id[int(n)] = v

    for k in convert:
        for_k = new[k]
        l = []
        for n in count():
            l.append(for_k.pop(n, {}))
            if not for_k: break
        new[k] = {'child_args':l}
    return new




class OrderedSet(list):
    """Set preserving item order."""

    def add(self, item):
        if item not in self:
            self.append(item)

    def add_all(self, iterable):
        for item in iterable:
            self.add(item)


class _RaiseAttributeError: pass

class ThreadSafeDescriptor(object):
    """Base class for threadsafe widget descriptors."""

class RequestLocalDescriptor(ThreadSafeDescriptor):
    """A descriptor that proxies to tw.framework.request_local for
    passing state among widgets in a thread-safe manner for the current request.

    Do not abuse this or code can become increasingly unmantainable!
    """
    def __init__(self, name, default=_RaiseAttributeError, __doc__=None, qualify_with_id=False):
        """Proxy to request local storage attribute named by ``name``.

        If ``default`` is provided no AttributeError will be raised if attribute
        does not exist ar request local storage when getting or deleting.
        """
        self.name = name
        self.default = default
        self.qualify_with_id = qualify_with_id
        self.__doc__ = __doc__ or "'%s' at request-local storage" % name


    def __get__(self, obj, typ=None):
        if obj is None:
            # If called on Widget class return descriptor
            return self
        import tw
        request_local = tw.framework.request_local
        name = self.name
        if self.qualify_with_id:
            name += "_%i" % id(obj)
        try:
            return getattr(request_local, name )
        except AttributeError:
            if self.default is _RaiseAttributeError:
                raise
            else:
                try:
                    value = self.default()
                except TypeError:
                    value = self.default
                setattr(request_local, name, value)
                return value

    def __set__(self, obj, value):
        import tw
        request_local = tw.framework.request_local
        name = self.name
        if self.qualify_with_id:
            name += "_%i" % id(obj)

        setattr(request_local, name, value)

    def __delete__(self, obj):
        import tw
        request_local = tw.framework.request_local
        name = self.name
        if self.qualify_with_id:
            name += "_%i" % id(obj)

        try:
            delattr(request_local, name)
        except AttributeError:
            if self.default is _RaiseAttributeError:
                raise


def install_framework(force=False):
    import tw
    if not hasattr(tw, 'framework') or force:
        from tw.core.registry import StackedObjectProxy, Registry
        from tw.mods import base
        dummy_framework = base.HostFramework()
        # start up a dummy request with dummy environ
        dummy_registry = Registry()
        dummy_registry.prepare()
        dummy_framework.start_request({
            'SCRIPT_NAME': '',
            'toscawidgets.prefix': '/toscawidgets',
            'paste.registry' : dummy_registry,
        })
        tw.framework = StackedObjectProxy(
            dummy_framework,
            name='ToscaWidgetsFramework',
            )

class Enum(dict):
    """Less strict Enum than the one provided by TG"""
    def __init__(self, *args):
        for arg in args:
            self[arg]= arg

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

class Bunch(dict):
    __setattr__ = dict.__setitem__

    def __delattr__(self, name):
        try:
            del self[name]
        except KeyError:
            raise AttributeError(name)

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError(name)

def make_bunch(d):
    """Converts a dict instance into a Bunch"""
    return Bunch(d)



# Adapted form turbogears'

class lazystring(object):
    """Has a number of lazily evaluated functions replicating a string.
    """

    def __init__(self, string):
        self.string = string

    def eval(self):
        from tw import framework
        return framework.translator(self.string)

    def __unicode__(self):
        return unicode(self.eval())

    def __str__(self):
        return str(self.eval())

    def __mod__(self, other):
        return self.eval() % other

    def __cmp__(self, other):
        return cmp(self.eval(), other)

    def __eq__(self, other):
        return self.eval() == other

class MultipleReplacer(object):
    """Performs several regexp substitutions on a string with a single pass.

    ``dct`` is a dictionary keyed by a regular expression string and with a
    callable as value that will get called to produce a subsituted value.

    The callable takes the matched text as first argument and may take any
    number of positional and keyword arguments. These arguments are any extra
    args passed when calling the instance.

    For performance, a single regular expression is built.

    Example::

        >>> string = "aaaabbbcc"
        >>> replacer = MultipleReplacer({
        ...     'a+':lambda g, context: g + context['after_a'],
        ...     'b+':lambda g, context: g + context['after_b'],
        ...     'c+':lambda g, context: context['before_c'] + g,
        ... })
        >>> replacer("aaaabbbcc", dict(
        ...     after_a = "1",
        ...     after_b = "2",
        ...     before_c = "3"
        ...     ))
        'aaaa1bbb23cc'
    """
    def __init__(self, dct, options=0):
        self._raw_regexp = r"|".join("(%s)" % k for k in dct.keys())
        self._substitutors = dct.values()
        self._regexp = re.compile(self._raw_regexp, options)

    def __repr__(self):
        return "<%s at %d (%r)>" % (self.__class__.__name__, id(self),
                                    self._raw_regexp)

    def _subsitutor(self, *args, **kw):
        def substitutor(match):
            for substitutor, group in izip(self._substitutors, match.groups()):
                if group is not None:
                    return substitutor(group, *args, **kw)
        return substitutor

    def __call__(self, string, *args, **kw):
        return self._regexp.sub(self._subsitutor(*args, **kw), string)


def iwarn(iterable, message, category=None, stacklevel=1):
    """Make an iterator that run warnings.warn(message, category, stacklevel) 
    right before the first value from iterator is returned.
    """
    warnings.warn(message, category, stacklevel)
    for x in iterable:
        yield x

# Stolen from Mako 
class LRUCache(dict):
    """A dictionary-like object that stores a limited number of items, discarding
    lesser used items periodically.

    this is a rewrite of LRUCache from Myghty to use a periodic timestamp-based
    paradigm so that synchronization is not really needed.  the size management 
    is inexact.
    """

    class _Item(object):
        def __init__(self, key, value):
            self.key = key
            self.value = value
            self.timestamp = time.time()
        def __repr__(self):
            return repr(self.value)

    def __init__(self, capacity, threshold=.5):
        self.capacity = capacity
        self.threshold = threshold

    def __getitem__(self, key):
        item = dict.__getitem__(self, key)
        item.timestamp = time.time()
        return item.value

    def values(self):
        return [i.value for i in dict.values(self)]
    
    def setdefault(self, key, value):
        if key in self:
            return self[key]
        else:
            self[key] = value
            return value
                
    def __setitem__(self, key, value):
        item = dict.get(self, key)
        if item is None:
            item = self._Item(key, value)
            dict.__setitem__(self, key, item)
        else:
            item.value = value
        self._manage_size()

    def _manage_size(self):
        while len(self) > self.capacity + self.capacity * self.threshold:
            bytime = dict.values(self)
            bytime.sort(lambda a, b: cmp(b.timestamp, a.timestamp))
            for item in bytime[self.capacity:]:
                try:
                    del self[item.key]
                except KeyError:
                    # if we couldnt find a key, most likely some other thread broke in 
                    # on us. loop around and try again
                    break

def disable_runtime_checks():
    """
    Disables runtime checks for possible programming errors regarding
    modifying widget attributes once a widget has been initialized.
    This option can significantly reduce Widget initialization time.
    NOTE: This operation modifies the Widget class and will affect any
        application using ToscaWidgets in the same process.
    """
    from tw.api import Widget
    del Widget.__setattr__
    del Widget.__delattr__

if __name__ == "__main__":
    import doctest
    doctest.testmod()



import unicodedata
from mako.runtime import Undefined

from cgi import escape
#from mako.filters import xml_escape


__all__ = ["attrs", "content"]

_BOOLEAN_ATTRS = frozenset(['selected', 'checked', 'compact', 'declare',
                            'defer', 'disabled', 'ismap', 'multiple',
                            'nohref', 'noresize', 'noshade', 'nowrap'])


def attrs(context, args=None, attrs=None):
    # Emulates Genshi's AttrsDirective (poorly)
    if isinstance(args, dict):
        args = args.items()
    if not args:
        args = []
    else:
        args = args[:]
    if attrs:
        args.extend(attrs.items())

    bools = _BOOLEAN_ATTRS

    return u" ".join([u'%s="%s"' % (k, escape(unicode(k in bools and k or v), True))
                      for k,v in args
                      if (k not in bools and v is not None) or (k in bools and v)])


def content(context, value):
    if value is None:
        return ""
    else:
        return escape(unicode(value))

def safe_str(context, value):
    """Converts value to its string representation, if unicode it makes it ascii
    safe by converting characters above 128 to their <128 equivalents."""
    if isinstance(value, unicode):
        return ''.join(unicodedata.normalize("NFD",c)[0] for c in value)
    else:
        return str(value)



import math
from warnings import warn
from inspect import isclass
import logging, re
from itertools import chain, count

import formencode
from formencode.foreach import ForEach
from formencode import Invalid

import tw
from tw.api import Widget, lazystring
from tw.core.util import iwarn
from tw.core import view
from tw.forms import (InputWidget, InputWidgetRepeater, validators)

log = logging.getLogger(__name__)

__all__ = [
    "FormField", "FormFieldRepeater", "ContainerMixin", "Form", "FieldSet",
    "TableMixin", "ListMixin", "ListForm", "ListFieldSet", "TextArea",
    "InputField", "TextField", "PasswordField", "HiddenField", "FileField",
    "Button", "SubmitButton", "ResetButton", "ImageButton", "SelectionField",
    "SingleSelectField", "SelectionList", "SingleSelectionMixin",
    "MultipleSelectionMixin", "MultipleSelectField", "RadioButtonList",
    "CheckBoxList", "TableForm", "CheckBox", "RadioButton",
    "SecureTicketField", "SecureFormMixin", "BooleanRadioButtonList",
    "TableFieldSet", "CheckBoxTable", "Spacer", "Label", "LabelHiddenField",
    ]

_ = lazystring

def name2label(name):
    """
    Convert a column name to a Human Readable name.

    Yanked from TGFastData
    """
    # Create label from the name:
    #   1) Convert _ to spaces
    #   2) Convert CamelCase to Camel Case
    #   3) Upcase first character of Each Word
    # Note: I *think* it would be thread-safe to
    #       memoize this thing.
    return ' '.join([s.capitalize() for s in
               re.findall(r'([A-Z][a-z0-9]+|[a-z0-9]+|[A-Z0-9]+)', name)])


class FormField(InputWidget):
    """
    Base class for all Widgets that can be attached to a Form or FieldSet.

    Form and FieldSets are in turn FormFields so they can be arbitrarily nested.
    These widgets can provide a validator that should validate and coerce the
    input they generate when submitted.
    """
    params = [
        "is_required", "label_text", "help_text", "attrs", "show_error",
        "disabled", "style", "container_attrs", "suppress_label",
        ]
    show_error = False
    show_error__doc = ("Should the field display it's own errors? Defaults to "
                       "False because normally they're displayed by the "
                       "container widget")
    disabled = None
    disabled__doc = ("Should the field be disbaled on render and it's input "
                     "ignored by the validator? UNIMPLEMENTED")
    attrs = {}
    attrs__doc = ("Extra attributes for the outermost DOM node")
    help_text = None
    help_text__doc = ("Description of the field to aid the user")
    label_text = None
    label_text__doc = "The text that should label this field"
    style = None
    style__doc = ("Style properties for the field. It's recommended to use "
                  "css classes and stylesheets instead of this parameter")
    container_attrs = {}
    container_attrs__doc = ("Extra attributes to include in the container tag "
                            "around this widget")
    suppress_label = False
    suppress_label__doc = ("Allows individual widgets to suppress the attached "
                           "label in their container")

    available_engines = ['mako', 'genshi']
    engine_name = 'genshi'

    @property
    def is_required(self):
        try:
            self.validate('', use_request_local=False)
            return False
        except (formencode.Invalid, KeyError):
            # Catch KeyError too since now  FieldsMatch raises it when
            # one of the fields it should match is missing. This is probably
            # A FE bug introduced in 1.1...
            return True
    is_required__doc = ("Computed flag indicating if input is required from "
                        "this field")

    file_upload = False

    def __init__(self, id=None, parent=None, children=[], **kw):
        super(FormField, self).__init__(id,parent,children, **kw)
        if self.label_text is None and self.name is not None:
            pos = self.name.rfind('.')
            name = self.name[pos+1:]
            self.label_text = name2label(name)


    def update_params(self,d):
        super(FormField, self).update_params(d)
        if self.is_required:
            d.css_classes.append('required')
        if d.disabled:
            d.attrs['disabled'] = 'disabled'

    def update_attrs(self, d, *args):
        """
        Fetches values from the dict and inserts the in the attrs dict.

        This is useful when you want to avoid boiler-place at the template:

        Instead of::

            <foo bar='$bar' zoo='$zoo' />

        Do::


            <foo py:attrs="attrs" />

        And inside update_params:

        .. code-block:: python

            self.update_attrs(d, 'bar', 'zoo')

        ('bar' and 'zoo' need to be listed at ``params``)

        """
        for name in args:
            d.setdefault('attrs',{}).setdefault(name, d[name])


class ContainerMixin(Widget):
    """
    A mix-in class for FormFields that contain other FormFields

    The following parameters are available:

    `show_children_errors`
      A flag indicating if the container should display it's children's errors

    It provides the template with these iterators:

    `fields`
      A list with all the container's visible FormFields present in `children`

    `hidden_fields`
      A list with all the container's hidden FormFields

    `ifields` (DEPRECATED: use `fields` instead)
      Iterates over all the container's visible FormFields present in `children`

    `ihidden_fields` (DEPRECATED: use `hidden_fields` instead)
      Iterates over all the container's hidden FormFields
    """
    params = ["show_children_errors"]
    show_error = False
    show_children_errors = True

    def __new__(cls, id=None, parent=None, children=[], **kw):
        fields = kw.pop('fields', None)
        if fields is not None:
            children = fields
        else:
            children = getattr(cls, 'fields', children)
        return super(ContainerMixin, cls).__new__(cls, id,parent,children,**kw)

    @property
    def ifields(self):
        return self.ifilter_children(
            lambda x: isinstance(x,FormField) and not isinstance(x,HiddenField)
            )

    @property
    def ihidden_fields(self):
        return self.ifilter_children(lambda x: isinstance(x,HiddenField))

    def _has_file_upload(self):
        for field in self.ifilter_children(
            lambda x: getattr(x, 'file_upload', False)
        ):
            return True
        return False

    def post_init(self, *args, **kw):
        log.debug("Setting 'file_upload' for %r", self)
        self.file_upload = self._has_file_upload()

    def update_params(self,d):
        super(ContainerMixin, self).update_params(d)
        d.fields = list(self.ifields)
        d.hidden_fields = list(self.ihidden_fields)

        d.ifields = iwarn(d.fields,
            "ifields is deprecated, use fields instead", DeprecationWarning, 2)
        d.ihidden_fields = iwarn(d.hidden_fields,
            "ihidden_fields is deprecated, use hidden_fields instead",
            DeprecationWarning, 2)


class FormFieldRepeater(InputWidgetRepeater, ContainerMixin, FormField):
    show_error = True   # Trick containers not to display its errors
    # Override engine stuff inherited from FormField
    available_engines = []
    engine_name = 'toscawidgets'


class Form(ContainerMixin, FormField):
    """
    A base class for all forms.

    Use this class as a base for your custom form. You should override it's
    template because it's a dummy one which does not display errors, help text
    or anything besides it's fields.

    The form will take care of setting its ``enctype`` if it contains any
    FileField
    """
    template = "tw.forms.templates.form"
    params = ["action", "method", "submit_text"]
    action = ''
    action__doc = "The url where the form's contents should be submitted"
    method = 'post'
    method__doc = "The HTTP request method to be used"
    submit_text = "Submit"
    submit_text__doc = ("Text that should appear in the auto-generated Submit "
                        "button. If None then no submit button will be "
                        "autogenerated.")
    submit_label_text = ''
    submit_label__doc = ("Label text for the auto-generated submit button. "
                         "If empty then no label will be generated.")

    def __init__(self, id=None, parent=None, children=[], **kw):
        super(Form, self).__init__(id, parent, children, **kw)
        if not hasattr(self.c, 'submit') and self.submit_text is not None:
            SubmitButton('submit', self, default=self.submit_text,
                         label_text=self.submit_label_text)

    def post_init(self, *args, **kw):
        log.debug("Setting 'enctype' for %r", self)
        if self._has_file_upload():
            self.attrs.setdefault('enctype', 'multipart/form-data')
        self.strip_name = kw.get('strip_name', self.is_root)

    def update_params(self, d):
        super(Form, self).update_params(d)
        d.method = d.method.lower()
        # Fails W3C validation if present
        d.attrs.pop('disabled', None)


class FieldSet(ContainerMixin, FormField):
    """
    Base class for a fieldset.

    Use this class for your custom fieldset. You should override it's template
    because it's a dummy one which does not display errors, help text or
    anything besides it's fields.
    """
    template = "tw.forms.templates.fieldset"
    params = ["legend"]
    legend__doc = ("The legend for the fieldset. If none is provided it will "
                   "use its name")

    def __init__(self, id=None, parent=None, children=[], **kw):
        super(FieldSet, self).__init__(id,parent,children, **kw)
        if self.legend is None:
            self.legend = self.label_text


class TableMixin(object):
    """
    Mix-in class for containers that use a table to render their fields
    """
    params = ["table_attrs", "show_labels", "hover_help"]
    table_attrs = {}
    show_labels = True
    hover_help = False

class ListMixin(object):
    """
    Mix-in class for containers that use a list to render their fields
    """
    params = ["list_attrs", "show_labels", "hover_help"]
    list_attrs = {}
    show_labels = True
    hover_help = False


class ListForm(Form, ListMixin):
    """
    A form that renders it's fields as an unordered list
    """
    template = "tw.forms.templates.list_form"

class TableForm(Form, TableMixin):
    """
    A form that renders it's fields in a table
    """
    template = "tw.forms.templates.table_form"


class ListFieldSet(FieldSet, ListMixin):
    """
    A fieldset that renders it's fields as an unordered list
    """
    template = "tw.forms.templates.list_fieldset"


class TableFieldSet(FieldSet, TableMixin):
    """
    A fieldset that renders it's fields in a table
    """
    template = "tw.forms.templates.table_fieldset"


class TextArea(FormField):
    """
    Displays a textarea.
    """
    params = ["rows", "cols"]
    rows__doc = "Number of rows to render"
    cols__doc = "Number of columns to render"
    template = "tw.forms.templates.textarea"
    rows = 7
    cols = 50

    def update_params(self,d):
        super(TextArea, self).update_params(d)
        self.update_attrs(d, "rows", "cols")


class InputField(FormField):
    """Base class for <input> fields"""
    params = ["type"]
    template = "tw.forms.templates.input_field"


class TextField(InputField):
    """A text field"""
    params = ["size", "max_size", "maxlength"]
    size__doc = "The size of the text field."
    maxlength__doc = "The maximum size of the field"
    max_size__doc = ("The maximum size of the field (DEPRECATED: use maxlength "
                     "instead)")
    type = "text"
    def update_params(self,d):
        super(TextField, self).update_params(d)
        if d.max_size is not None:
            d.maxlength = d.max_size
            warn("max_size is deprecated, use maxlength instead",
                 DeprecationWarning, 6)
        self.update_attrs(d, "size", "maxlength")


class PasswordField(InputField):
    """A password field."""
    type = "password"


class HiddenField(InputField):
    """A hidden field """
    type = "hidden"


class FileField(InputField):
    """A file upload field"""
    type = "file"
    file_upload = True

    def adapt_value(self, value):
        # This is needed because genshi doesn't seem to like displaying
        # cgi.FieldStorage instances
        return None


class Button(InputField):
    """A button field"""
    type = "button"
    params = ["named_button"]
    named_button = False

    def __init__(self, id=None, parent=None, children=[], **kw):
        super(Button, self).__init__(id,parent,children, **kw)
        if not self.named_button:
            self.validator = None

    def _full_name(self):
        if not self.named_button:
            return None
        return super(Button, self)._full_name()
    name = property(_full_name)


class SubmitButton(Button):
    type = "submit"

    def update_params(self,d):
        super(SubmitButton, self).update_params(d)
        # A submit button with and id of 'submit' will make
        # form.submit == 'object' in JS code. See ticket #1295
        if d.id == 'submit':
            d.id = None


class ResetButton(Button):
    type = "reset"


class ImageButton(Button):
    params = ["src", "width", "height", "alt"]
    type = "image"
    def update_params(self,d):
        super(ImageButton, self).update_params(d)
        if isinstance(d.get('src', self.src), tw.api.Link):
            d['src'] = d.get('src', self.src).link
        self.update_attrs(d, "src", "width", "height", "alt")


class CheckBox(InputField):
    type = "checkbox"
    validator = validators.Bool
    def update_params(self, d):
        super(CheckBox, self).update_params(d)
        try:
            checked = self.validator.to_python(d.value)
        except Invalid:
            checked = False
        d.attrs['checked'] = checked or None


class RadioButton(InputField):
    type = "radio"


class SelectionField(FormField):
    selected_verb = None
    params = ["options"]
    options = []

    def update_params(self, d):
        super(SelectionField, self).update_params(d)
        grouped_options = []
        options = []
        d['options'] = self._iterate_options(d['options'])
        # Coerce value if possible so _is_options_selected can compare python
        # values. This is needed when validation fails because FE will send
        # uncoerced values.
        value = self.safe_validate(d['value'])
        for optgroup in d["options"]:
            if isinstance(optgroup[1], (list,tuple)):
                group = True
                optlist = optgroup[1][:]
            else:
                group = False
                optlist = [optgroup]
            for i, option in enumerate(self._iterate_options(optlist)):
                if len(option) is 2:
                    option_attrs = {}
                elif len(option) is 3:
                    option_attrs = dict(option[2])
                if self._is_option_selected(option[0], value):
                    option_attrs[self.selected_verb] = self.selected_verb
                optlist[i] = (self.adjust_value(option[0]), option[1],
                              option_attrs)
            options.extend(optlist)
            if group:
                grouped_options.append((optgroup[0], optlist))
        # options provides a list of *flat* options leaving out any eventual
        # group, useful for backward compatibility and simpler widgets
        d["options"] = options
        if grouped_options:
            d["grouped_options"] = grouped_options
        else:
            d["grouped_options"] = [(None, options)]


    def _iterate_options(self, options):
        for option in options:
            if not isinstance(option, (tuple,list)):
                yield (option, option)
            else:
                yield option


class SelectionList(ListMixin, SelectionField):
    params = ["field_type", "id_counter"]
    selected_verb = "checked"
    template = "tw.forms.templates.selection_list"

    def id_counter(self):
        return count(0)


class SingleSelectionMixin(object):
    def _is_option_selected(self, option_value, value):
        return option_value == value


class MultipleSelectionMixin(object):
    def _is_option_selected(self, option_value, value):
        return value is not None and option_value in value

    def post_init(self, *args, **kw):
        # Only override the user-provided validator if it's not a ForEach one,
        # which usually means the user needs to perform validation on the list
        # as a whole.
        self._original_validator = self.validator
        if not (isinstance(self.validator, ForEach) or
                (isclass(self.validator) and
                 issubclass(self.validator, ForEach))):
            self.validator = ForEach(self.validator)


    def adjust_value(self, value, validator=None):
        # Our ForEach validator will return a single element list. We only
        # want that single element
        return SelectionField.adjust_value(self, value,
                                           self._original_validator)


class SingleSelectField(SingleSelectionMixin, SelectionField):
    selected_verb = 'selected'
    template = "tw.forms.templates.select_field"


class MultipleSelectField(MultipleSelectionMixin, SelectionField):
    params = ["size"]
    size = 5
    selected_verb = 'selected'
    template = "tw.forms.templates.select_field"

    def update_params(self,d):
        super(MultipleSelectField, self).update_params(d)
        self.update_attrs(d, "size")
        d['attrs']['multiple'] = True


class RadioButtonList(SingleSelectionMixin, SelectionList):
    field_type = "radio"


class BooleanRadioButtonList(RadioButtonList):
    options = [(False, _("No")), (True, _("Yes"))]
    validator = validators.StringBoolean


class CheckBoxList(MultipleSelectionMixin, SelectionList):
    field_type = "checkbox"


class SecureTicketField(HiddenField):
    """
    Hidden form field that offers some protection against Cross-Site
    Request Forgery:

        http://en.wikipedia.org/wiki/Cross-site_request_forgery

    This protection is not complete against XSS or web browser bugs, see

        http://www.cgisecurity.com/articles/csrf-faq.shtml

    A per-session per-form authentication key is generated, and
    injected to this hidden field. They are compared on form
    validation. On mismatch, validation error is displayed.
    """

    def __init__(self, id=None, parent=None, children=[],
                 session_secret_cb=None, **kw):
        """Initialize the CSRF form token field.

        session_secret_cb() should return (session_secret, token)
        where session_secret is a random per-session secret string,
        and token some string associated with the current user.
        """
        super(SecureTicketField, self).__init__(id, parent, children, **kw)
        self.validator = validators.SecureTicketValidator(self,
                                                          session_secret_cb)

    def update_params(self, d):
        super(SecureTicketField, self).update_params(d)
        d['value'] = self.validator.get_hash()
        return d


class SecureFormMixin(FormField):
    """
    Protect against Cross-site request forgery, by adding
    SecureTicketField to the form.

    This can be use like this::

        class MyForm(ListForm, SecureFormMixin):
            ...

        def session_secret_cb():
            "Return session-specific secret data string and some data.
             Both of them should stay constant per form per user."
            secret = session['secret']
            user_id = c.user.user_name
            return str(secret), str(user_id)

        form = MyForm(session_secret_cb=session_secret_cb)

    or::

        class MyForm(ListForm, SecureFormMixin):

            ...

            def session_secret_cb():
                ...

        form = MyForm()
    """

    def post_init(self, *args, **kw):
        session_secret_cb = kw.pop('session_secret_cb', None)
        #super(SecureFormMixin, self).__init__(id, parent, children, **kw)
        if hasattr(self, 'session_secret_cb') and session_secret_cb is None:
            session_secret_cb = self.session_secret_cb
        SecureTicketField("form_token__", self,
                          session_secret_cb=session_secret_cb)


def group(seq, size):
    if not hasattr(seq, 'next'):
        seq = iter(seq)
    while True:
        chunk = []
        try:
            for i in xrange(size):
                chunk.append(seq.next())
            yield chunk
        except StopIteration:
            if chunk:
                yield chunk
            break


class CheckBoxTable(CheckBoxList):
    """
    A checkboxlist that renders a table of checkboxes of num_cols columns
    """
    template = "tw.forms.templates.check_box_table"
    params = ["num_cols"]
    num_cols = 1

    def update_params(self, d):
        super(CheckBoxTable, self).update_params(d)
        d.options_rows = group(d.options, d.num_cols)
        d.grouped_options_rows = [(g, group(o, d.num_cols)) for g, o in d.grouped_options]


class Spacer(FormField):
    """
    A widget to insert spacing within a form
    """
    template = "tw.forms.templates.spacer"
    validator = None


class Label(FormField):
    """
    A textual label
    """
    params = ['text']
    text = ''
    template = "tw.forms.templates.label"
    validator = None
    suppress_label = True


class LabelHiddenField(InputField):
    """A hidden field with a label showing its contents"""
    template = "tw.forms.templates.label_hidden"




import errno
import re
import operator
import shutil
import sys
import os
import tempfile
import subprocess
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

import pkg_resources
from setuptools import Command

from tw.core.resources import registry

class archive_tw_resources(Command):
    """Setuptools command to copy and optionally compress all static resources
    from a series of distributions and their dependencies into a directory
    where they can be served by a fast web server.

    To enable compression of CSS and JS files you will need to have installed a
    Java Runtime Environment and YUICompressor
    (http://www.julienlecomte.net/yuicompressor)

    In order for resources from widget eggs to be properly collected these
    need to have a 'toscawidgets.widgets' 'widgets' entry-point which points
    to a module which, when imported, instantiates all needed JS and CSS Links.

    The result is laid out in the output directory in such a way that when
    a a web server such as Apache or Nginx is configured to map URLS that
    begin with /toscawidgets to that directory static files will be served
    from there bypassing python completely.


    To integrate this command into your build process you can add these lines
    to ``setup.cfg``::

        [archive_tw_resources]
        output = /home/someuser/public_html/toscawidgets/
        compresslevel = 2
        distributions = MyProject
        yuicompressor = /home/someuser/bin/yuicompressor.jar
        onepass = true

        [aliases]
        deploy = archive_tw_resources --force install

    This way you can run::
        
        $ python setup.py deploy

    To install a new version of your app and copy/compress resources.
    """
    description = "Copies ToscaWidgets static resources into a directory where"\
                  " a fast web-server can serve them."
    user_options = [
        ("output=", "o",
         "Output directory. If it doesn't exist it will be created."),
        ("force", "f", "If output dir exists, it will be ovewritten"),
        ("onepass", None, "If given, yuicompressor will only be called once "\
                          "for each kind of file with a all files "\
                          "together and then separated back into smaller "\
                          "files"),
        ("compresslevel=", "c",
         "Compression level: 0) for no compression (default). "\
                            "1) for js-minification. "\
                            "2) for js & css compression"),
        ("yuicompressor=", None, "Name of the yuicompressor jar."),
        ("distributions=", "d",
         "List of widget dists. to include resources from "
         "(dependencies will be handled recursively). Note that "
         "these distributions need to define a 'toscawidgets.widgets' "
         "'widgets' entrypoint pointing to a a module where "
         "resources are located."),
        ]

    def initialize_options(self):
        self.output = ''
        self.force = False
        self.onepass = False
        self.compresslevel = 0
        self.distributions = []
        self.yuicompressor = 'yuicompressor.jar'

    def finalize_options(self):
        self.ensure_string("output")
        self.ensure_string("yuicompressor")
        self.ensure_string_list("distributions")
        self.compresslevel = int(self.compresslevel)
        self.yuicompressor = os.path.abspath(self.yuicompressor)

    def run(self):
        if not self.output:
            print >> sys.stderr, "Need to specify an output directory"
            return
        if not self.distributions:
            print >> sys.stderr, "Need to specify at least one distribution"
            return
        if os.path.exists(self.output) and not self.force:
            print >> sys.stderr, ("Destination dir %s exists. " % self.output)+\
                                  "Use -f to ovewrite"
            return
        if self.compresslevel > 0 and not os.path.exists(self.yuicompressor):
            print >> sys.stderr, "Could not find YUICompressor at " + \
                                 self.yuicompressor
            return

        tempdir = tempfile.mktemp()
        self.execute(os.makedirs, (tempdir,), "Creating temp dir %s" % tempdir)
        if self.compresslevel > 0:
            if self.onepass:
                self.writer = OnePassCompressingWriter(self, tempdir)
            else:
                self.writer = CompressingWriter(self, tempdir)
        else:
            self.writer = FileWriter(self, tempdir)

        self.execute(self._copy_resources, tuple(), "Extracting resources")
        self.writer.finalize()
        if os.path.exists(self.output):
            self.execute(shutil.rmtree, (self.output,),
                         "Deleting old output dir %s" % self.output)
        self.execute(os.makedirs, (self.output,), "Creating output dir")
        final_dest = os.path.join(self.output, registry.prefix.strip('/'))
        self.execute(shutil.move, (tempdir, final_dest),
                     "Moving build to %s" % final_dest)

    def _load_widgets(self, distribution):
        try:
            requires = [r.project_name for r in
                        pkg_resources.get_distribution(distribution).requires()]
            map(self._load_widgets, requires)
            mod = pkg_resources.load_entry_point(distribution,
                                                 'toscawidgets.widgets',
                                                 'widgets')
            self.announce("Loaded %s" % mod.__name__)
        except ImportError:
            self.announce("%s has no widgets entrypoint" % distribution)

    def _copy_resources(self):
        map(self._load_widgets, self.distributions)
        for webdir, dirname in registry:
            parts = filter(None, webdir.split('/'))
            modname = parts[0]
            fname = '/'.join(parts[1:])
            self.execute(self._copy_resource_tree, (modname, fname),
                         "Copying %s recursively into %s" % 
                         (dirname, self.writer.base))


    def _copy_resource_tree(self, modname, fname):
        try:
            for name in pkg_resources.resource_listdir(modname, fname):
                name = '/'.join((fname, name))
                rel_name = '/'.join((modname, name))
                if pkg_resources.resource_isdir(modname, name):
                    self.execute(self._copy_resource_tree, (modname, name),
                                 "Recursing into " + rel_name)
                else:
                    stream = pkg_resources.resource_stream(modname, name)
                    filename = '/'.join((modname, name))
                    self.execute(self.writer.write_file, (stream, filename),
                                 "Processing " + filename)
                    stream.close()
        except OSError, e:
            if e.errno == errno.ENOENT:
                self.warn("Could not copy %s" % repr((modname, fname, e)))


class FileWriter(object):
    def __init__(self, cmd, base):
        self.base = base
        self.cmd = cmd

    def finalize(self):
        pass

    def write_file(self, stream, path):
        final = os.path.join(self.base, path)
        if not os.path.exists(os.path.dirname(final)):
            os.makedirs(os.path.dirname(final))
        dest = open(final, 'wb')
        self.announce("Writting %s" % path)
        shutil.copyfileobj(stream, dest)
        dest.close()


    # Delegate methods to Command
    for name in "warn announce error execute".split():
        exec """\
def %(name)s(self, *args, **kw):
    return self.cmd.%(name)s(*args, **kw)
""" % locals()

class CompressingWriter(FileWriter):

    def __init__(self, *args, **kw):
        super(CompressingWriter, self).__init__(*args, **kw)
        self.counters = 0, 0

    def finalize(self):
        try:
            avg =  reduce(operator.truediv, self.counters) * 100
            msg = "Total JS&CSS compressed size is %.2f%% of original" % avg
            self.announce(msg)
        except ZeroDivisionError:
            # No files were compressed
            pass

    def compress(self, stream, path):
        typ = path.split('.')[-1]
        if typ not in ('css', 'js'):
            return stream
        args = ['java', '-jar', self.cmd.yuicompressor, '--type', typ]
        if self.cmd.compresslevel < 2:
            args.append('--nomunge')
        args.append('--charset=utf8')
        p = subprocess.Popen(args, stdout=subprocess.PIPE,
                             stdin=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        self.announce("Compressing %s" % path)
        buffer = StringIO()
        shutil.copyfileobj(stream, buffer)
        data = buffer.getvalue()
        if not data:
            return buffer
        stdout, stderr = p.communicate(data)
        if p.returncode != 0:
            self.warn("Failed to compress %s: %d" % (path, p.returncode))
            self.warn("File will be copied untouched")
            sys.stderr.write(stderr)
            sys.stderr.write(stdout)
            stream.seek(0)
        else:
            count = len(stdout), len(data)
            ratio = reduce(operator.truediv, count)
            self.counters = map(sum, zip(self.counters, count))
            msg = "Compressed %s (New size: %.2f%%)" % (path, ratio*100)
            self.announce(msg)
            stream = StringIO(stdout)
        return stream

    def write_file(self, stream, path):
        stream = self.compress(stream, path)
        return super(CompressingWriter, self).write_file(stream, path)

class OnePassCompressingWriter(CompressingWriter):
    def __init__(self, *args, **kw):
        super(OnePassCompressingWriter, self).__init__(*args, **kw)
        #XXX This comment trick only works with JS as of YUICompressor 2.3.5
        self._caches = {'js': StringIO()}
        self._marker = "/*! MARKER #### %(path)s #### MARKER */"
        regexp = r"^\/\* MARKER #### (?P<path>.*?) #### MARKER \*\/$"
        self._re = re.compile(regexp)

    def _demultiplex(self, stream):
        cur_file = None
        buffer = StringIO()
        stream.seek(0)
        for line in stream:
            m = self._re.match(line)
            if m:
                if cur_file:
                    buffer.seek(0)
                    FileWriter.write_file(self, buffer, cur_file)
                    buffer.truncate(0)
                cur_file = m.group('path')
            else:
                buffer.write(line)

    def finalize(self):
        self.announce("Compressing all defered files")
        for typ, cache in self._caches.iteritems():
            cache.seek(0)
            # self.compress only wants to know the file extension to see
            # what kind of file it is, we pass a dummy one
            compressed = self.compress(cache, '__defered__.'+typ)
            self._demultiplex(compressed)
        super(OnePassCompressingWriter, self).finalize()

    def write_file(self, stream, path):
        typ = path.split('.')[-1]
        cache = self._caches.get(typ)
        if not cache:
            self.announce("Will not consider %s for onepass" % path)
            return CompressingWriter.write_file(self, stream, path)
        print >> cache, self._marker % locals()
        self.announce("Defering %s for compression in one pass" % path)
        shutil.copyfileobj(stream, cache)



from datetime import date
__all__ = [
    '__VERSION__',
    '__COPYRIGHT__',
    '__AUTHOR__',
    '__LICENSE__',
    '__PACKAGE_NAME__',
    ]

__PACKAGE_NAME__ = "tw.forms"
__VERSION__ = "0.9.7.2"
__AUTHOR__ = "Alberto Valverde Gonzalez"
__EMAIL__ = "alberto@toscat.net"
__COPYRIGHT__ = "2006-%d Alberto Valverde Gonzalez and contributors" % date.today().year
__LICENSE__ = "MIT"



from tw.api import WidgetsList, CSSSource, JSSource, js_function
from tw.forms import *
from tw.forms.validators import *
from formencode.national import USPostalCode as PostalCode, \
                                USStateProvince as StateProvince

__all__ = ["AddUserForm"]

# Lil' chunk o' CSS for tasty eye-candy ;)
# External css files can be wrapped with CSSLink
css = CSSSource("""
label.required, .fielderror {
    font-weight: bold;
    color: red;
};
""")


# We define the source for some JS functions we're going to interface
# External js files can be wrapped with JSLink
functions = JSSource("""
var focus_element = function (elem) {
    var elem = document.getElementById(elem);
    elem.focus(); elem.select();
    }; 
    """,
    )

alert = js_function('alert')
focus_element = js_function('focus_element')


# This is needed because of the way TurboGears validates as it adds
# spurious variables to the value being validated.
class FilteringSchema(Schema):
    filter_extra_fields = True
    allow_extra_fields = True


# Now the form widgets....


class AddressFieldset(ListFieldSet):
    class fields(WidgetsList):
        street = TextField(validator=UnicodeString)
        number = TextField(validator=Int, size=4)
        zip_code = TextField(validator=PostalCode())
        state = TextField(default='NY',validator=StateProvince())

    validator = FilteringSchema

class AddUserForm(ListForm):
    class fields(WidgetsList):
        id = HiddenField(default="I'm hidden!")
        name = TextField(
            validator = UnicodeString(not_empty=True), 
            default = "Your name here"
            )
        gender = RadioButtonList(
            options = "Male Female".split(),
            )
        age = SingleSelectField(
            validator = Int, 
            options = range(100)
            )
        email = TextField(
            validator = Email()
            )
        date = CalendarDateTimePicker()
        roles = CheckBoxList(
            options = "Manager Admin Editor User".split(),
            )
        groups = MultipleSelectField(
            options = "Group1 Group2 Group3".split(),
            )
        password = PasswordField(
            validator = String(not_empty=True), 
            max_size = 10
            )
        password_confirm = PasswordField(
            validator = String(not_empty=True), 
            max_size=10
            )
        # We wrap the address fieldset with a FormFieldRepeater to handle
        # repetitions. This can be done with *any* FormField.
        address = FormFieldRepeater(
            widget = AddressFieldset(), 
            repetitions = 2, 
            max_repetitions = 5
            )

    # allow adding js calls dynamically for a request
    include_dynamic_js_calls = True

    css = [css]
    javascript = [functions]
    validator = FilteringSchema(
        chained_validators = [FieldsMatch('password','password_confirm')],
        )


    def update_params(self, d):
        super(AddUserForm, self).update_params(d)
        # Focus and select the 'name' field on the form
        # The adapter we just wrote lets us pass formfields as parameters and
        # the right thing will be done.
        if not d.error:
            self.add_call(focus_element(d.c.name))
        else:
            self.add_call(
                alert('The form contains invalid data\n%s'% unicode(d.error))
                )

class DemoSingleSelect(SingleSelectField):
    options = [
        "Python",
        "Haskell",
        "Java",
        "Ruby",
        "Erlang",
        "Javascript"
        ]

class DemoMultipleSelect(MultipleSelectField):
    options = [
        "Python",
        "Haskell",
        "Java",
        "Ruby",
        "Erlang",
        "Javascript"
        ]

class DemoCheckBoxList(CheckBoxList):
    options = [
        "Python",
        "Haskell",
        "Java",
        "Ruby",
        "Erlang",
        "Javascript"
        ]

class DemoRadioButtonList(RadioButtonList):
    options = [
        "Python",
        "Haskell",
        "Java",
        "Ruby",
        "Erlang",
        "Javascript"
        ]

class DemoCheckBoxTable(CheckBoxTable):
    num_cols = 2
    options = [
        "Python",
        "Haskell",
        "Java",
        "Ruby",
        "Erlang",
        "Javascript"
        ]

class Person(object):
    def __init__(self, name, age):
        self.__dict__.update(locals())

class DemoDataGrid(DataGrid):
    fields = [("Name", "name"), ("Age","age")]
    default = [
        Person('Lucy', 29),
        Person('Peter', 15),
        Person('Tiffany', 17),
        ]



"""
.. testoutput::
   :hide:

   >>> # This is so the doctests start from a clean state
   >>> from tw.core.util import install_framework; install_framework(True)

ToscaWidgets can inject resources that have been registered for injection in
the current request.

Usually widgets register them when they're displayed and they have instances of
:class:`tw.api.Resource` declared at their :attr:`tw.api.Widget.javascript` or
:attr:`tw.api.Widget.css` attributes.

Resources can also be registered manually from a controller or template by
calling their :meth:`tw.api.Resource.inject` method.

When a page including widgets is rendered, Resources that are registered for
injection arre collected in request-local
storage area (this means any thing stored here is only visible to one single
thread of execution and that its contents are freed when the request is
finished) where they can be rendered and injected in the resulting html.

ToscaWidgets' middleware can take care of injecting them automatically (default)
but they can also be injected explicitly, example::
    

   >>> from tw.api import JSLink, inject_resources
   >>> JSLink(link="http://example.com").inject()
   >>> html = "<html><head></head><body></body></html>"
   >>> inject_resources(html)
   '<html><head><script type="text/javascript" src="http://example.com"></script></head><body></body></html>'

Once resources have been injected they are popped from request local and
cannot be injected again (in the same request). This is useful in case
:class:`injector_middleware` is stacked so it doesn't inject them again.

Injecting them explicitly is neccesary if the response's body is being cached
before the middleware has a chance to inject them because when the cached
version is served no widgets are being rendered so they will not have a chance
to register their resources.
"""
import re
import logging

from webob import Request

from tw import framework
from tw.core.util import MultipleReplacer

log = logging.getLogger(__name__)


__all__ = ["inject_resources", "injector_middleware"]

def injector_middleware(app):
    """
    Middleware that injects resources (if any) into the page whenever the
    output is html (peeks into Content-Type header).

    Normally you don't have to stack thi yourself because
    :class:`tw.core.middleware.ToscaWidgetsMiddleware` does it when it is
    passed the ``inject_resources`` flag as True (default).
    """
    def _injector(environ, start_response):
        req = Request(environ)
        resp = req.get_response(app)
        content_type = resp.headers.get('Content-Type','text/plain').lower()
        if 'html' in content_type:
            resources = framework.pop_resources()
            if resources:
                resp.body = inject_resources(resp.body, resources, resp.charset)
        return resp(environ, start_response)
    return _injector


class _ResourceInjector(MultipleReplacer):
    def __init__(self):
        return MultipleReplacer.__init__(self, {
            r'<head.*?>': self._injector_for_location('head'),
            r'</head.*?>': self._injector_for_location('headbottom', False),
            r'<body.*?>': self._injector_for_location('bodytop'),
            r'</body.*?>': self._injector_for_location('bodybottom', False)
            }, re.I|re.M)

    def _injector_for_location(self, key, after=True):
        def inject(group, resources, encoding):
            inj = u'\n'.join([r.render() for r in resources.get(key, [])])
            inj = inj.encode(encoding)
            if after:
                return group + inj
            return  inj + group
        return inject

    def __call__(self, html, resources=None, encoding=None):
        """Injects resources, if any, into html string when called.

        .. note::
           Ignore the ``self`` parameter if seeing this as
           :func:`tw.core.resource_injector.inject_resources` docstring
           since it is an alias for an instance method of a private class.

        ``html`` must be a ``encoding`` encoded string. If ``encoding`` is not
        given it will be tried to be derived from a <meta>.
        
        Resources for current request can be obtained by calling
        ``tw.framework.pop_resources()``. This will remove resources
        from request and a further call to ``pop_resources()`` will return an
        empty dict.
        """
        if resources is None:
            resources = framework.pop_resources()
        if resources:
            # Only inject if there are resources registered for injection
            encoding = encoding or find_charset(html) or 'ascii'
            html = MultipleReplacer.__call__(self, html, resources, encoding)
        return html


# Bind __call__ directly so docstring is included in docs
inject_resources = _ResourceInjector().__call__


_charset_re = re.compile(r"charset\s*=\s*(?P<charset>[\w-]+)([^\>])*",
                         re.I|re.M)
def find_charset(string):
    m = _charset_re.search(string)
    if m:
        return m.group('charset').lower()



import re, weakref, logging
from copy import copy
from warnings import warn
from itertools import ifilter, count, chain, izip, islice, ifilterfalse
from inspect import isclass

import tw
from util import (assert_bool_attr, callable_wo_args,
                  unflatten_args, OrderedSet, make_bunch,
                  install_framework, ThreadSafeDescriptor,
                  RequestLocalDescriptor, LRUCache)

install_framework()

import view
from exceptions import *
from meta import WidgetType, WidgetsList

__all__ = [
    "Widget", 
    "WidgetsList", 
    "WidgetRepeater", 
    "RepeatedWidget",
    "WidgetBunch", 
    "RepeatingWidgetBunch",
    "Child",
    "valid_id",
    "adapt_value",
    ]


log = logging.getLogger(__name__)



only_if_initialized = assert_bool_attr(
    '_is_initialized', True, WidgetUninitialized
    )
only_if_uninitialized = assert_bool_attr(
    '_is_initialized', False, WidgetInitialized
    )

def only_if_unlocked(func):
    def __setattr__(self, k, *args):
        descriptor = getattr(self.__class__, k, None)
        if not isinstance(descriptor, ThreadSafeDescriptor) and self._is_locked:
            raise WidgetLocked
        func(self, k, *args)
    return __setattr__

valid_id_re = re.compile(r'^[a-zA-Z][\w\-\_\:\.]*$')
_deprecated_id_re = re.compile(r'^\w+$')



def valid_id(s):
    if valid_id_re.match(s):
        return True
    elif s == '_method':
        #ignore this one small hack to help pages use HTTP verbs
        return True
    elif _deprecated_id_re.match(s):
        warn("The id %s will no longer be supported since it doesn't conform "
             "to the W3C Spec: http://www.w3.org/TR/xhtml1/#C_8" % s,
             DeprecationWarning, 3)
        return True
    else:
        return False

serial_generator = count()


class Widget(view.Renderable):
    """
    Base class for all widgets. 

    Example:

    .. sourcecode:: python

        >>> w = Widget('foo')
        >>> w.id
        'foo'

        >>> w = Widget('foo', children=[Widget('c1'), Widget('c2')])
        >>> [c.id for c in w.children]
        ['foo_c1', 'foo_c2']
        >>> [c.parent.id for c in w.children]
        ['foo', 'foo']

    It is a **must** that all initial state is entirely determined by the 
    arguments to this function. This means that two widgets 
    (of the same class) that receive the same parameters
    must behave in exactly the same way. You should not rely on external
    sources inside __init__ to set initial state. 
    
    If you need to fetch data from external sources, do it at 
    :meth:`update_params` instead.

    Essential pre, and post initialization is done in :meth:`__new__` and 
    :meth:`post_init` respectively. :meth:`post_init` is guaranteed to run after
    the instance finishes initialization and it's behavior is rather special
    as all post_init's in mro will be called to have a chance to set final
    state in the instance.

    Basic pre-initialization consists of binding all kw arguments to the
    widget instance, attaching the widget to it's parent (if given),
    attaching the children and copying mutable arguments listed at
    :attr:`params` from the class to the instance to avoid accidental
    manipulation.

    .. sourcecode:: python

        >>> w = Widget('foo', a=1, b=2)
        >>> w.id
        'foo'
        >>> w.a 
        1
        >>> w.b
        2

    Basic post-initialization consists of caching required CSS and JS 
    resources and setting the widget as initialized preventing further
    modification of it's attributes.
        
    .. sourcecode:: python

        >>> w = Widget('foo', a='1', b='2')
        >>> w.a = 'bar'
        Traceback (most recent call last):
            ...
        WidgetLocked: The widget is locked. It's unthread-safe to alter it's attributes after initialization.

    Widget attributes can only be modified in this method because widgets
    should behave in a state-less way as they are shared among threads for
    multiple requests.

    Per request modification of variables sent to the template should be 
    done inside :meth:`update_params` and all state flowing from parent to
    children should occur inside that dict.

    Widgets should be instantiated at import time and reused among requests,
    most widgets allow overriding most of their parameters (not neccesarily all
    of them) at display time to change behavior. You should try avoiding
    instantiating widgets for every request as their initialization could be
    quite expensive for heavily nested widgets.
    
    Request-local storage provided by the hosting framework in 
    ``tw.framework.request`` can be used to pass state among widgets 
    which don't share the same root.
    
    """
    __metaclass__ = WidgetType

    parent = None
    default = None
    params = {
        'id': ('The id of this widget. This id is used to reference a widget '
               'from its parent ``children`` attribute and is usually the '
               'DOM id of outermost HTML tag of the widget.'),
        'css_class': 'Main CSS class for this widget',
        'css_classes': 'A list with extra css classes for the widget.'
        }
    css = []
    javascript = []
    css_classes = []

    _is_initialized = False
    _is_locked = False

    def displays_on(self):
        if self.is_root:
            return tw.framework.default_view
        else:
            return self.parent.engine_name
    displays_on = property(displays_on, doc="""\
        Where the widget is being displayed on
        """)
    
    #XXX: Some of these properties could be implemented as static attributes
    def id(self):
        return '_'.join(reversed(
            [w.id_path_elem for w in self.path if w.id_path_elem]
            )) or None
    id = property(id, doc="""\
        The calculated id of the widget. This string will provide a unique
        id for each widget in the tree in a format which allows to re-recreate
        the nested structure.
        Example::

            >>> A = Widget("A", children=[
            ...     Widget("B", children=[
            ...         Widget("C")
            ...         ])
            ...     ])
            ...
            >>> C = A.c.B.c.C
            >>> C.id
            'A_B_C'

        """)

    def key(self):
        return '.' + '.'.join(reversed(
            [w.id_path_elem for w in self.path if w.id_path_elem][:-1]
            )) or None
    key = property(key, doc="""\
        A string that can be used as a key to index the dictionary of
        parameters sent to the root widget so it reaches this widget when
        displaying.

        Example::

            >>> A = Widget("A", children=[
            ...     Widget("B", children=[
            ...         Widget("C")
            ...         ])
            ...     ])
            ...
            >>> C = A.c.B.c.C
            >>> C.key
            '.B.C'
        """)


    def path(self):
        item = self
        while item:
            yield item
            item = item.parent
    path = property(path, doc="""\
        Iterates a walk from this widget to the root of the tree
        """)


    @property
    def id_path_elem(self):
        return self._id


    def root(self):
        return list(self.path)[-1]
    root = property(root, doc="The root of this widget tree")

    def is_root(self):
        return self.parent is None
    is_root = property(is_root, doc="True if the widget doesn't have a parent")

    def __new__(cls, id=None, parent=None, children=[], **kw):
        """
        Takes care of Widget instances creation. Should not need to be overrien
        likely. 
        """
        obj = view.Renderable.__new__(cls,id,parent,children,**kw)
        # The previous version of this used the __dict__ attribute to update
        # things, but that doesn't fire any fset properties, and can actually
        # lose data if a property is set, the following will not behave like
        # that
        for k,v in kw.iteritems():
            if not k.startswith('_'):
                try:
                    setattr(obj, k, v)
                except AttributeError, e:
                    #skip setting the value of a read only property
                    pass
        obj.orig_kw = kw.copy()
        
        if id is not None and not valid_id(id):
            raise ValueError("%r is not a valid id for a Widget"%id)
        obj._id = id
        obj._serial = serial_generator.next()

        # Attach the widget to its parent
        if parent is not None:
            if parent._is_initialized:
                raise WidgetInitialized
            obj.parent = weakref.proxy(parent)
            obj.parent.children.append(obj)

        # Append children passed as args or defined in the class, former 
        # override later
        obj.c = obj.children = WidgetBunch()
        if isclass(children) and issubclass(children, WidgetsList):
            children = children()
        if not [obj._append_child(c) for c in children]:
            cls_children = cls._cls_children
            if isclass(cls_children) and issubclass(cls_children, WidgetsList):
                cls_children = cls_children()
            [obj._append_child(c) for c in cls_children]



        # Copy mutable attrs from __class__ into self, if not found in self 
        # set to None
        for name in chain(cls.params, ['css', 'javascript']):
            try:
                attr = getattr(obj, name, None)
                if isinstance(attr, (list,dict)):
                    attr = copy(attr)
                setattr(obj, name, attr)
            except AttributeError: 
                # In case we try to set a read-only property
                pass
                
        # Initialize the static js calls list
        obj._js_calls = []

        # Initialize resources OrderedSet
        obj._resources = OrderedSet()

        # Set default css class for the widget
        if not getattr(obj, 'css_class', None):
            obj.css_class = obj.__class__.__name__.lower()

        return obj

    @only_if_initialized
    def clone(self, *args, **kw):
        """
        Returns a cloned version of the widget instance, optionally
        overriding initialization parameters.

        This is the only way to safely "modify" a widget instance.

        Example::

            >>> w = Widget('foo', a=2)
            >>> w.id, w.a
            ('foo', 2)
            >>> w2 = w.clone(a=3)
            >>> w2.id, w2.a
            ('foo', 3)

        """
        #log.debug("Cloning %r", self)
        return Child(
            self.__class__, self._id, children=self.children, **self.orig_kw
            )(*args, **kw)


    @only_if_initialized
    def _as_repeated(self, *args, **kw):
        cls = self.__class__
        new_name = 'Repeated'+cls.__name__
        new_class = type(new_name, (RepeatedWidget, cls), {})
        #log.debug("Generating %r for repeating %r", new_class, self)
        return Child(
            new_class, self._id, children=self.children, **self.orig_kw
            )(*args, **kw)




    def __init__(self, id=None, parent=None, children=[], **kw):
        """Initializes a Widget instance.

        `id` 
            The widget's id. All widgets in the same level of nested
            widgets trees should have distinct ids.

        `parent`
            A reference to the widget's parent. This parent needs to be in an
            uninitialized state which means it can only be passed to a child
            inside the parent's __init__ method.

        `children`
            A list, or WidgetsList (instance or class) or any other iterable 
            with a reference to the children this widget should have.

        `\*\*kw`
            Any other extra keyword arguments for the widget. All of these will 
            get bound to the Widget instance.

        """
        # we need a dummy init here for the metaclass to wrap
        pass



    def _collect_resources(self):
        """picks up resources from self and all children"""
        oset = self._resources
        oset.add_all(chain(*[c._resources for c in self.css]))
        oset.add_all(chain(*[c._resources for c in self.javascript]))
        oset.add_all(chain(*[c._resources for c in self.children]))
        


    def post_init(self, *args, **kw):
        """
        This method is called for all :class:`tw.api.Widget` base classes to
        perform final setup after the widget is initialized but before it is
        locked.
        """
        if len(self._js_calls) > 0:
            from tw.core.resources import JSFunctionCalls
            #log.debug("Creating JSFunctionCalls for %r. Functions: %s",
                #self, self._js_calls,
                #)
            self.javascript.append(
                JSFunctionCalls(function_calls=self._js_calls)
                )
        self._collect_resources() 
        #log.debug("Finished initializing %r", self)
        assert not self._is_initialized
        self._is_initialized = True
        assert not self._is_locked
        self._is_locked = True


    def walk(self, filter=None, recur_if_filtered=True):
        """
        Does a pre-order walk on widget tree rooted at self optionally 
        applying a filter on them.
        
        Example::
    
            >>> W = Widget
            >>> w = W('a', children=[W('b', children=[W('c')]), W('d')])
            >>> ''.join(i._id for i in w.walk())
            'abcd'
            >>> ''.join(i._id for i in w.walk(lambda x: not x.is_root))
            'bcd'
            >>> ''.join(i._id for i in w.walk(lambda x: x._id == 'c'))
            'c'

            Recursion can be prevented on children that not match filter.

            >>> ''.join(i._id for i in w.walk(lambda x: x._id == 'c', False))
            ''
            
        """
        def _walk():
            yield self
            iterator = iter(self.children)
            if filter and not recur_if_filtered:
                iterator = self.ifilter_children(filter)
            for c in iterator:
                for w in c.walk(filter, recur_if_filtered):
                    yield w
        if filter:
            return ifilter(filter, _walk())
        return _walk()

    def add_call(self, call, location="bodybottom"):
        """
        Adds a :func:`tw.api.js_function` call that will be made when the 
        widget is rendered.
        """
        if self._is_initialized:
            #log.debug("Adding call <%s> for %r dynamically.", call, self)
            from tw.core.resources import dynamic_js_calls
            dynamic_js_calls.inject_call(call, location)
        else:
            #log.debug("Adding call <%s> for %r statically.", call, self)
            self._js_calls.append(str(call))
            




    @only_if_initialized
    def retrieve_css(self):
        warn("retrieve_css is deprecated. Please use retrieve_resources "
             "instead and filter them yourself", DeprecationWarning, 2)
        return []

    @only_if_initialized
    def retrieve_javascript(self):
        warn("retrieve_javascript is deprecated. Please use retrieve_resources "
             "instead and filter them yourself", DeprecationWarning, 2)
        return []

    @only_if_initialized
    def retrieve_resources(self):
        """
        Returns a dict keyed by location with ordered collections of
        resources from this widget and its children as values.
        """
        from tw.api import locations
        resources = dict((k, OrderedSet()) for k in locations)
        for r in self._resources:
            resources[r.location].add(r)
        return resources


    def adapt_value(self, value):
        """
        Adapt object *value* for rendering in this widget. Should return one of:
         * A list of objects for repeated widgets.
         * A dict for widgets with children, keyed by the children's ids.
         * Any other object the widget understands.
         """
        # Handle MultiDict instances
        if hasattr(value, 'dict_of_lists'):
            for k, v in value.dict_of_lists().items():
                if len(v) == 1:
                    value[k] = v[0]
                else:
                    value[k] = v
        # If we have children, create a dict from the attributes of value
        elif len(self.children_deep) > 0 and not isinstance(value, (dict,list)):
            value = dict([
                (w._id, getattr(value, w._id)) 
                    for w in self.children_deep if w._id and hasattr(value, w._id)
                ])
        return value

    def register_resources(self):
        """
        Register the resources required by this Widget with
        :attr:`tw.framework` for inclusion in the page.

        This method is called whenever a :class:`Widget` is rendered
        """
        if self.is_root:
            tw.framework.register_resources(self.retrieve_resources())

    def render(self, value=None, **kw):
        """
        Renders a widget as an unicode string.
        """
        kw = self.prepare_dict(value, kw)
        self.register_resources()
        return super(Widget, self).render(**kw)

    def display(self, value=None, **kw):
        """
        Renders a widget and adapts the output. This method **must** be used
        to display child widgets inside their parent's template so output is
        adapted.
        
        Unlike :meth:`tw.api.Widget.render`, :meth:`tw.api.Widget.display`
        returns adapted output compatible with the template the widget is being
        rendered on. For example, this is needed so Genshi doesn't autoescape
        string output from mako and to serialize Genshi output on the other way
        around.
        """
        kw = self.prepare_dict(value, kw)
        self.register_resources()
        return super(Widget, self).display(**kw)

    def __call__(self, value=None, **kw):
        return self.display(value, **kw)

    def get_default(self):
        """Returns the default value for the widget. If the default is a funtion
        that it can be called without arguments it will be called on each
        render to retrieve a value"""
        if callable_wo_args(self.default):
            value = self.default()
        else:
            value = self.default
        return value

    @property
    def children_deep(self):
        return self.children

    def prepare_dict(self, value, d, adapt=True):
        """
        Prepares the all kw arguments sent to `display` or `render` before
        passing the kw argument's dict to `update_params`.
        """
        if value is None:
            value = self.get_default()
        if adapt:
            d['value'] = self.adapt_value(value)
        else:
            d['value'] = value

        # Move args passed to child widgets into child_args
        child_args = d.setdefault('child_args', {})
        for k in d.keys():
            if '.' in k:# or '-' in k:
                child_args[k.lstrip('.')] = d.pop(k)

        d['args_for'] = self._get_child_args_getter(child_args)
        d['value_for'] = self._get_child_value_getter(d['value'])
        d['c'] = d['children'] = self.children
        d = make_bunch(d)
        self.update_params(d)
        s = set([d['css_class'],])
        classes = d['css_classes']
        for item in classes:
            s.add(item)
        # Compute the final css_class string
        d['css_class']= ' '.join(s)
        # reset the getters here so update_params has a chance to alter
        # the arguments to children and the value
        d['args_for'] = self._get_child_args_getter(d['child_args'])
        d['value_for'] = self._get_child_value_getter(d.get('value'))
        # Provide a shortcut to display a child field in the template
        d['display_child'] = self._child_displayer(self.children,
                                                   d['value_for'],
                                                   d['args_for'])
        return d


    def update_params(self, d):
        """
        Updates the dict sent to the template for the current request.

        It is called when displaying or rendering a widget with all keyword
        arguments passed stuffed inside dict.

        Widget subclasses can call super cooperatively to avoid
        boiler-plate code as `Widget.update_params` takes care of pre-populating
        this dict with all attributes from self listed at `params` 
        (copying them if mutable) and preparing arguments for child widgets.

        Any parameter sent to `display` or `render` will override those fetched
        from the instance or the class.

        Any function listed at `params` which can be called without arguments
        will be automatically called to fetch fresh results on every request. 
        Parameters not found either on the class, the instance or the keyword 
        args to `display` or `render` will be set to None.
        
        .. sourcecode:: python

            >>> class MyWidget(Widget):
            ...     params = ["foo", "bar", "null"]
            ...     foo = "foo"
            ...
            >>> w = MyWidget('test', bar=lambda: "bar")
            >>> d = {}
            >>> w.update_params(d)
            >>> d['bar']
            'bar'
            >>> d['foo']
            'foo'
            >>> d['null'] is None
            True
            >>> d = {'foo':'overriden'}
            >>> w.update_params(d)
            >>> d['foo']
            'overriden'

        """
        # Populate dict with attrs from self listed at params
        for k in ifilterfalse(d.__contains__, self.params):
            attr = getattr(self, k, None)
            if attr is not None:
                if isinstance(attr, (list, dict)):
                    attr = copy(attr)
                # Variables that are callable with no args are automatically
                # called here
                elif not isinstance(attr, Widget) and callable_wo_args(attr):
                    log.debug("Autocalling param '%s'", k)
                    attr = attr()
            d[k] = attr






    def ifilter_children(self, filter):
        """
        Returns an iterator for all children applying a filter to them.

        .. sourcecode:: python

            >>> class Widgets(WidgetsList):
            ...     aa = Widget()
            ...     ab = Widget()
            ...     ba = Widget()
            ...     bb = Widget()
            ...
            >>> w = Widget(children=Widgets)
            >>> [c.id for c in w.ifilter_children(lambda w: w.id.startswith('a'))]
            ['aa', 'ab']

        """
        return ifilter(filter, self.children)






    def _get_child_value_getter(self, value):
        def value_getter(child_id):
            if value:
                if (hasattr(child_id, 'repetition') and 
                    isinstance(value,list)
                ):
                    child_id = child_id.repetition
                elif isinstance(child_id, Widget) and isinstance(value,dict):
                    child_id = child_id._id
                try:
                    return value[child_id]
                except (IndexError,KeyError,TypeError):
                    None
        return value_getter


    def _get_child_args_getter(self, child_args):
        if isinstance(child_args, dict):
            child_args = unflatten_args(child_args)
        def args_getter(child_id):
            if (hasattr(child_id, 'repetition') and 
                isinstance(child_args, list)
            ):
                child_id = child_id.repetition
            elif (isinstance(child_id, Widget) and 
                isinstance(child_args, dict)
            ):
                child_id = child_id._id
            try:
                return child_args[child_id]
            except (IndexError,KeyError,TypeError):
                return {}
        return args_getter



    @only_if_uninitialized
    def _append_child(self,obj):
        """Append an object as a child"""
        if isinstance(obj, Widget):
            obj._append_to(self)
        elif isinstance(obj, Child):
            obj(self)
        else:
            raise ValueError("Can only append Widgets or Childs, not %r" % obj)


    @only_if_initialized
    def _append_to(self, parent=None):
        return self.clone(parent)



    @only_if_unlocked
    def __setattr__(self,k,v):
        object.__setattr__(self,k,v)


    @only_if_unlocked
    def __delattr__(self,k):
        object.__delattr__(self,k)


    def __repr__(self):
        name = self.__class__.__name__
        return "%s(%r, children=%r, **%r)" % (
            name, self._id, self.children, self.orig_kw
            )

    def __str__(self):
        return self.render()

    def __ne__(self, other):
        return not (self == other)

    def __eq__(self, other):
        return (
          (getattr(other, '__class__', None) is self.__class__) and 
          # Check _id so ancestors are not taken into account
          (other._id == self._id) and 
          (other.children == self.children) and (other.orig_kw == self.orig_kw)
          )

    @staticmethod
    def _child_displayer(children, value_for, args_for):
        def display_child(widget, **kw):
            if isinstance(widget, (basestring,int)):
                widget = children[widget]
            child_kw = args_for(widget)
            child_kw.update(kw)
            return widget.display(value_for(widget), **child_kw)
        return display_child

# Available as a public symbol for easier extension with PEAK-Rules for
# backwards compatibility
adapt_value = Widget.adapt_value.im_func




class Child(object):
    """
    Prepares a Widget to being attached to a parent Widget.

    Creates a Widget instance with supplied arguments to the constructor when
    called (optionally overriding default arguments).
    
        >>> c = Child(Widget, 'foo')
        >>> w = c()
        >>> w.id
        'foo'

    Parameters can be overriden when called.

        >>> w = c(id='bar')
        >>> w.id
        'bar'

    """
    __slots__ = ("widget_class", "id", "children", "kw")

    def __init__(self, widget_class, id=None, children=[], **kw):
        self.widget_class, self.id  = widget_class, id, 
        self.children, self.kw = children, kw

    def __call__(self, parent=None, **kw):
        kw_ = self.kw.copy()
        kw_.update(id=self.id, parent=parent, children=self.children)
        kw_.update(kw)
        return self.widget_class(**kw_)





#XXX: Should enhance and clean up so setting widgets is not allowed if the
#     WidgetBunch is used as a widget's 'children' attribute and the widget
#     is already initialized. BTW, this WidgetBunch_attrs thingie is becomming
#     ugly ... should re-implement properly someday...
_WidgetBunch_attrs = frozenset(['_widget_lst', '_widget_dct', '_widget',
                                '_repetitions', '_parent', '_repetition_cache'])
class WidgetBunch(object):
    """
    An ordered collection of widgets. 

        >>> from tw.core import Widget 
        >>> wb = WidgetBunch(Widget('foo'), Widget('bar'))
        >>> wb[0].id == 'foo'
        True
        >>> wb[1].id == 'bar'
        True

    Exposes a mixed dict/list interface which permits indexing both by widget 
    id and position.

        >>> wb['foo'] == wb[0]
        True
        >>> wb['bar'] == wb[1]
        True

    Also permits attribute access as long as the attribute doesn't conflict
    with an internal attribute, in case of conflict the internal attrbute
    will prevail.

        >>> wb.foo.id == 'foo'
        True
        >>> wb.append(Widget('append'))
        >>> wb.append.id 
        Traceback (most recent call last):
            ...
        AttributeError: 'function' object has no attribute 'id'
        >>> wb['append'].id == 'append'
        True

    Iteration is also supported

        >>> [w.id for w in wb]
        ['foo', 'bar', 'append']

    Some dict-like iterators too

        >>> [id for id in wb.iterkeys()]
        ['foo', 'bar', 'append']
        >>> [id for id in wb.keys()]
        ['foo', 'bar', 'append']
        >>> [w.id for w in wb.itervalues()]
        ['foo', 'bar', 'append']
        >>> [w.id for w in wb.values()]
        ['foo', 'bar', 'append']

    """

    def __init__(self, *args):
        self._widget_lst = []
        self._widget_dct = {}
        for wid in args:
            self.append(wid)

    def append(self,wid):
        self.__setattr__(wid._id, wid)

    def retrieve_css(self):
        oset = OrderedSet()
        for child in self:
            oset.add_all(child.retrieve_css())
        return oset

    def retrieve_javascript(self):
        oset = OrderedSet()
        for child in self:
            oset.add_all(child.retrieve_javascript())
        return oset

    def retrieve_resources(self):
        from tw.core.resources import merge_resources, locations
        resources = dict((k, OrderedSet()) for k in locations)
        for w in self:
            merge_resources(resources, w.retrieve_resources())
        return resources

    def __getitem__(self, item):
        if isinstance(item, basestring):
            return self._widget_dct[item]
        elif isinstance(item, int):
            return self._widget_lst[item]
        raise KeyError, "No widget by %r" % item

    def __getattr__(self, name):
        if name in _WidgetBunch_attrs:
            return object.__getattribute__(self,name)
        try:
            return self[name]
        except KeyError:
            raise AttributeError, "No widget by %r" % name

    def __setattr__(self, name, value):
        # Note that this setattr allows name = None so append can use it.
        if name in _WidgetBunch_attrs:
            return object.__setattr__(self,name,value)
        self._widget_lst.append(value)
        self._widget_dct[name] = value

    def __iter__(self):
        for wid in self._widget_lst:
            yield wid

    def __ne__(self, other):
        return not (self==other)

    def __eq__(self, other):
        try:
            if len(self) == len(other):
                for a,b in izip(other,self):
                    if a!=b: return False
                return True
        except TypeError: pass
        return False

    def __getslice__(self,start=0,stop=-1,step=1):
        return islice(self,start,stop,step)

    def __len__(self):
        return len(self._widget_lst)

    def __nonzero__(self):
        return bool(len(self))

    def __contains__(self,item):
        dct = self._widget_dct
        try:
            return dct[item._id] == item
        except KeyError:
            return False

    def keys(self):
        return list(self.iterkeys())

    def iterkeys(self):
        for w in self: yield w._id

    def values(self):
        return list(self.itervalues())

    itervalues = __iter__

    def __repr__(self):
        return `self._widget_lst`


class RepeatingWidgetBunch(WidgetBunch):
    _repetitions = 0
    _widget = None

    def __init__(self, *args, **kw):
        super(RepeatingWidgetBunch, self).__init__(*args, **kw)
        self._repetition_cache = LRUCache(100)
    def __len__(self):
        return self._repetitions

    def __iter__(self, reps=None):
        if reps is None:
            reps = self._repetitions
        for i in xrange(reps):
            yield self[i]
    
    def __getitem__(self, item):
        if not isinstance(item, int):
            raise KeyError("Must specify an integer")
        try:
            rep = self._repetition_cache[item]
        except KeyError:
            rep = self._widget.clone(repetition=item)
            # Work around widget's locking mechanism since it is thread-safe
            # to do it *here*. Note that the parent is not directly aware that
            # it has this repeated widget as a child and it doesn't care
            # really since I create those children on the fly
            object.__setattr__(rep, 'parent', self._parent)
            self._repetition_cache[item] = rep
        return rep










class WidgetRepeater(Widget):
    params = ["repetitions", "extra"]
    widget = None
    repetitions = 3
    max_repetitions = None
    extra = 0
    id_path_elem = None

    template = "$output"

    @property
    def key(self):
        raise AttributeError("A WidgetRepeater has no meaningful key")


    def __new__(cls, id=None, parent=None, children=[], **kw):
        widget = kw.get('widget', getattr(cls, 'widget', None))
        if widget is None:
            warn("No repeated widget specified in %s" % cls.__name__)
            widget = Widget()
        if isclass(widget) and issubclass(widget, Widget):
            widget = widget()
        obj = super(WidgetRepeater, cls).__new__(cls, id,parent,**kw)
        obj.c = obj.children = RepeatingWidgetBunch()
        obj.c._parent = weakref.proxy(obj)
        obj.c._repetitions = kw.get('repetitions', cls.repetitions)
        obj.c._widget = widget._as_repeated(obj, id=obj._id)
        return obj


    def update_params(self, d):
        super(WidgetRepeater,self).update_params(d)
        if d.get('max_repetitions', self.max_repetitions) is not None:
            warn("max_repetitions is deperecated and no longer has any effect",
                 DeprecationWarning, 3)
        v_f = d['value_for']
        a_f = d['args_for']
        outputs = [
            w.render(v_f(w), isextra=(w.repetition >= len(d['value'])), **a_f(w)) for w in d['children'].__iter__(
                max(d['repetitions'], len(d['value']) + d.extra))
            ]
        d["output"] = '\n'.join(o for o in outputs if o)    








class RepeatedWidget(Widget):
    params = ["repetition"]
    repetition = 0

    @property
    def id_path_elem(self):
        return '%s-%d' % (self._id, self.repetition or 0)

    @only_if_initialized
    def _as_repeated(self, *args, **kw):
        return self.clone(*args, **kw)















if __name__ == "__main__":
    import doctest
    doctest.testmod(optionflags = doctest.ELLIPSIS|doctest.NORMALIZE_WHITESPACE)



from exceptions import *
from meta import *
from util import *
from view import *
from base import *
from resources import *
from resource_injector import *
from js import *
from middleware import make_middleware
from tw.mods.base import *



import re
import logging
from inspect import isclass
from copy import copy
from itertools import ifilter

from formencode import Invalid, FancyValidator
from formencode.schema import Schema
from formencode.foreach import ForEach
from formencode.variabledecode import NestedVariables, variable_decode

from tw.api import (Widget, WidgetRepeater, RequestLocalDescriptor,
                    RepeatedWidget, RepeatingWidgetBunch)
from tw.core.base import only_if_initialized, Child


__all__ = ["InputWidget", "InputWidgetRepeater", "merge_schemas"]

log = logging.getLogger(__name__)


class DefaultValidator(FancyValidator): pass



#------------------------------------------------------------------------------
# Base class for all widgets that can generate input for the app
#------------------------------------------------------------------------------

valid_name = re.compile(r'^[\w\_\:]*$').match


class InputWidget(Widget):
    params = dict(
        name = "Name for this input Widget. This is the name of the variable "\
               "that will reach the controller. This parameter can only be "\
               "set during widget initialization",
        strip_name = "If this flag is True then "\
                     "the name of this widget will not be included in the "\
                     "fully-qualified names of the widgets in this subtree. "\
                     "This is useful to 'flatten-out' nested structures. "\
                     "This parameter can only be set during initialization."
        )
    validator = DefaultValidator
    # If this is True, validator's which are Schema subclasses/instances will
    # be called to adjust value with from_python.
    # This is off by default because it's usually not the desired behavior
    # since Schemas usually call their subvalidators' from_python recursively
    # and that will convert the whole form causing errors when the widgets'
    # children try to adjust the value themselves. Activate only if you know
    # what you're doing.
    force_conversion = False
    strip_name = False

    def __new__(cls, id=None, parent=None, children=[], **kw):
        obj = super(InputWidget, cls).__new__(cls, id, parent, children, **kw)
        obj._name = kw.pop('name', id)
        if obj._name and not valid_name(obj._name):
            raise ValueError("%s is not a valid name for an InputWidget" %
                             obj._name)
        return obj


    @only_if_initialized
    def _as_repeated(self, *args, **kw):
        cls = self.__class__
        new_name = 'Repeated'+cls.__name__
        new_class = type(new_name, (RepeatedInputWidget, cls), {})
        log.debug("Generating %r for repeating %r", new_class, self)
        return Child(
            new_class, self._id, children=self.children, **self.orig_kw
            )(*args, **kw)

    @property
    def name_path_elem(self):
        if self.strip_name:
            return None
        else:
            return self._name

    def _full_name(self):
        return '.'.join(reversed([w.name_path_elem for
            w in self.path if getattr(w, 'name_path_elem', None)])) or None
    name = property(_full_name)


    error_at_request = RequestLocalDescriptor('error',
        __doc__ = """Validation error for current request.""",
        default = None,
        qualify_with_id=True,
        )

    value_at_request = RequestLocalDescriptor('value',
        __doc__ = """Value being validated in current request.""",
        default = None,
        qualify_with_id=True,
        )

    def adapt_value(self, value):
        value = super(InputWidget, self).adapt_value(value)
        if value == '':
            # This is needed when an ancestor's Schema has converted blank input
            # for us into a blank string.
            value = None
        # Work around formencode.schema.Schema which doesn't
        # run UnicodeString sub-validators on encoded strings when
        # validation fails. This causes Genshi to choke when redisplaying
        # a failed form.
        elif isinstance(value, str):
            if getattr(self.validator, 'inputEncoding', None) \
              and isinstance(self.validator.inputEncoding, str):
                value = unicode(value, self.validator.inputEncoding)
            elif getattr(self.validator, 'encoding', None) \
              and isinstance(self.validator.encoding, str):
                value = unicode(value, self.validator.encoding)
        return value

    #XXX: use_request_local should default to False but that needs patching
    #     TG 1.0. If implementing this for the first time do not depend on this
    #     default and provide it explicitly so your code won't break when this
    #     changes.
    def validate(self, value, state=None, use_request_local=True):
        """Validate value using validator if widget has one. If validation fails
        a formencode.Invalid exception will be raised.

        If ``use_request_local`` is True and validation fails the exception and
        value will be placed at request local storage so if the widget is
        redisplayed in the same request ``error`` and ``value`` don't have to
        be passed explicitly to ``display``.
        """
        if self.validator:
            try:
                value =  self.validator.to_python(value, state)
            except Invalid, error:
                if use_request_local:
                    self.error_at_request = error
                    # Check for 'items' to support MultiDicts et al.
                    if hasattr(value, 'items'):
                        value = variable_decode(value)
                    self.value_at_request = value
                raise
        return value

    def adjust_value(self,value, validator=None):
        """
        Adjusts the python value sent to :meth:`InputWidget.display` with
        the validator so it can be rendered in the template.
        """
        validator = validator or self.validator
        if validator and ((not isinstance(self.validator, Schema)) or
                           self.force_conversion):
            # Does not adjust_value with Schema because it will recursively
            # call from_python on all sub-validators and that will send
            # strings through their update_params methods which expect
            # python values. adjust_value is called just before sending the
            # value to the template, not before.
            # This behaviour can be overriden with the force_conversion flag
            try:
                value = validator.from_python(value)
            except Invalid:
                # Ignore conversion errors so bad-input is redisplayed
                # properly
                pass
            if value is None:
                # A None will skip renderingthe value attribute altogether in
                # Genshi templates and that's not what we want. Worse still,
                # String templates will render a "None", yuck! Convert it
                # into an empty string if the validator hasn't done it already.
                value = ""
        return value

    def safe_validate(self, value):
        """Tries to coerce the value to python using the validator. If
        validation fails the original value will be returned unmodified."""
        try:
            value = self.validate(value, use_request_local=False)
        except Exception:
            pass
        return value

    @property
    def children_deep(self):
        out = []
        for c in self.children:
            if getattr(c, 'strip_name', False):
                out += c.children_deep
            else:
                out.append(c)
        return out

    def prepare_dict(self, value, kw, adapt=True):
        """
        Prepares the dict sent to the template with functions to access the
        children's errors if any.
        """
        if value is None:
            value = self.get_default()
        if adapt:
            value = self.adapt_value(value)
        if self.is_root:
            error = kw.setdefault('error', self.error_at_request)
        else:
            error = kw.setdefault('error', None)
        if error:
            if self.children:
                self.propagate_errors(kw, error)
            if self.is_root:
                value_at_request = self.value_at_request
                if isinstance(value, dict) and isinstance(value_at_request, dict):
                    value.update(value_at_request)
                else:
                    value = value_at_request

        if not isinstance(self.validator, (ForEach,Schema)):
            # Need to coerce value in case the form is being redisplayed with
            # uncoereced value so update_params always deals with python
            # values. Skip this step if validator will recursively validate
            # because that step will be handled by child widgets.
            value = self.safe_validate(value)
        # Propagate values to grand-children with a name stripping parent
        for c in self.children:
            if getattr(c, 'strip_name', False):
                for subc in c.children_deep:
                    if hasattr(subc, '_name'):
                        try:
                            v = value.pop(subc._name)
                        except KeyError:
                            pass
                        else:
                            value.setdefault(c._name, {})[subc._name] = v
        kw['error_for'] = self._get_child_error_getter(kw['error'])
        kw = super(InputWidget, self).prepare_dict(value, kw, adapt=False)
        kw['field_for'] = _field_getter(self.c)
        # Provide backwards compat. for display_field_for. should deprecate
        kw['display_field_for'] = kw['display_child']
        # Adjust the value with the validator if present and the form is not
        # being redisplayed because of errors *just before* sending  it to the
        # template.
        if not error:
            kw['value'] = self.adjust_value(kw['value'])
            # Rebind these getters with the adjusted value
            kw['value_for'] = self._get_child_value_getter(kw.get('value'))
            # Provide a shortcut to display a child field in the template
            kw['display_child'] = self._child_displayer(self.children,
                                                        kw['value_for'],
                                                        kw['args_for'])
        return kw

    def update_params(self, d):
        super(InputWidget, self).update_params(d)
        if d.error:
            d.css_classes.append("has_error")

    def propagate_errors(self, parent_kw, parent_error):
        child_args = parent_kw.setdefault('child_args',{})
        if parent_error.error_dict:
            if self.strip_name:
                for c in self.children:
                    for subc in c.children:
                        if hasattr(subc, '_name'):
                            try:
                                e = parent_error.error_dict.pop(subc._name)
                            except KeyError:
                                continue
                            if c._name not in parent_error.error_dict:
                                inv = Invalid("some error", {}, e.state, error_dict={})
                                parent_error.error_dict[c._name] = inv
                            child_errors = parent_error.error_dict[c._name].error_dict
                            child_errors[subc._name] = e
            for k,v in parent_error.error_dict.iteritems():
                child_args.setdefault(k, {})['error'] = v


    def _get_child_error_getter(self, error):
        def error_getter(child_id):
            try:
                if error and error.error_list:
                    if (isinstance(child_id, Widget) and
                       hasattr(child_id, 'repetition')
                    ):
                        child_id = child_id.repetition
                    return error.error_list[child_id]
                elif error and error.error_dict:
                    if isinstance(child_id, Widget):
                        child_id = child_id._id
                    return error.error_dict[child_id]
            except (IndexError,KeyError): pass
            return None
        return error_getter


    def post_init(self, *args, **kw):
        """
        Takes care of post-initialization of InputWidgets.
        """
        self.generate_schema()

    def generate_schema(self):
        """
        If the widget has children this method generates a `Schema` to validate
        including the validators from all children once these are all known.
        """
        if _has_child_validators(self) and not isinstance(self, WidgetRepeater):
            if isinstance(self.validator, Schema):
                log.debug("Extending Schema for %r", self)
                self.validator = _update_schema(_copy_schema(self.validator),
                                                self.children)
            elif isclass(self.validator) and issubclass(self.validator, Schema):
                log.debug("Instantiating Schema class for %r", self)
                self.validator = _update_schema(self.validator(), self.children)
            elif self.validator is DefaultValidator:
                self.validator = _update_schema(Schema(), self.children)

            if self.is_root and hasattr(self.validator, 'pre_validators'):
                #XXX: Maybe add VariableDecoder to every Schema??
                log.debug("Appending decoder to %r", self)
                self.validator.pre_validators.insert(0, VariableDecoder)
            for c in self.children:
                if c.strip_name:
                    v = self.validator.fields.pop(c._id)
                    merge_schemas(self.validator, v, True)




class InputWidgetRepeater(WidgetRepeater, InputWidget):
    name_path_elem = None

    def propagate_errors(self, parent_kw, parent_error):
        child_args = parent_kw.setdefault('child_args',[])
        # The error we get at this point doesn't have an error_list, it's
        # buried a few levels deep, so recurse until we find it, or at least
        # until we've inspected it all.
        if parent_error.error_dict and not parent_error.error_list:
            for k, v in parent_error.error_dict.iteritems():
                self.propagate_errors(parent_kw, v)
            return
        if parent_error.error_list:
            for i,e in enumerate(parent_error.error_list):
                try:
                    child_args[i]['error'] = e
                except IndexError:
                    child_args.append({'error':e})


    def post_init(self, *args, **kw):
        if self.validator is DefaultValidator and _has_child_validators(self):
            log.debug("Generating a ForEach validator for %r", self)
            self.validator = ForEach(self.children[0].validator)

    def adjust_value(self, value, validator=None):
        # no-op as value will be adjusted by repeated widgets
        return value

class RepeatedInputWidget(RepeatedWidget):
    _label_text = None
    @property
    def name_path_elem(self):
        return "%s-%d" % (self._name, self.repetition or 0)

    def set_label_text(self, val):
        # jtate- I'm not really proud of this, I'd much rather pass a template
        # in, but since the only parameter sent from the repeater to the
        # repeated widget on instantiation is the repetition count, this will
        # have to do for now
        rep = self.repetition or 0
        label = val.replace(str(rep), '#%d' % (rep+1))
        self._label_text = label
        return self._label_text

    def get_label_text(self):
        # self.__dict__ can have a label_text set, via Widget.__new__ kw args,
        # but the property is not used to set it, sync it on first access
        #if self._label_text is None:
        #    if self.__dict__.get('label_text') is not None:
        #        self._label_text = self.__dict__.pop('label_text')
        return self._label_text
    label_text = property(get_label_text, set_label_text)

#------------------------------------------------------------------------------
# Automatic validator generation functions.
#------------------------------------------------------------------------------


def _has_validator(w):
    try:
        return w.validator is not None
    except AttributeError:
        return False


def _has_child_validators(widget):
    for w in widget.children:
        if _has_validator(w): return True
    return False



def _copy_schema(schema):
    """
    Does a deep copy of a Schema instance
    """
    new_schema = copy(schema)
    new_schema.pre_validators = copy(schema.pre_validators)
    new_schema.chained_validators = copy(schema.chained_validators)
    new_schema.order = copy(schema.order)
    fields = {}
    for k, v in schema.fields.iteritems():
        if isinstance(v, Schema):
            v = _copy_schema(v)
        fields[k] = v
    new_schema.fields = fields
    return new_schema

def _update_schema(schema, children):
    """
    Extends a Schema with validators from children. Does not clobber the ones
    declared in the Schema.
    """
    for w in ifilter(_has_validator, children):
        _add_field_to_schema(schema, w._name, w.validator)
    return schema

def _add_field_to_schema(schema, name, validator):
    """ Adds a validator if any to the given schema """
    if validator is not None:
        if isinstance(validator, Schema):
            # Schema instance, might need to merge 'em...
            if name in schema.fields:
                assert (isinstance(schema.fields[name], Schema),
                        "Validator for '%s' should be a Schema subclass" % name)
                validator = merge_schemas(schema.fields[name], validator)
            schema.add_field(name, validator)
        elif _can_add_field(schema, name):
            # Non-schema validator, add it if we can...
            schema.add_field(name, validator)
    elif _can_add_field(schema, name):
        schema.add_field(name, DefaultValidator)

def _can_add_field(schema, field_name):
    """
    Checks if we can safely add a field. Makes sure we're not overriding
    any field in the Schema. DefaultValidators are ok to override.
    """
    current_field = schema.fields.get(field_name)
    return bool(current_field is None or
                isinstance(current_field, DefaultValidator))

def merge_schemas(to_schema, from_schema, inplace=False):
    """
    Recursively merges from_schema into to_schema taking care of leaving
    to_schema intact if inplace is False (default).
    """
    if not inplace:
        to_schema = _copy_schema(to_schema)

    # Recursively merge child schemas
    is_schema = lambda f: isinstance(f[1], Schema)
    seen = set()
    for k, v in ifilter(is_schema, to_schema.fields.iteritems()):
        seen.add(k)
        from_field = from_schema.fields.get(k)
        if from_field:
            v = merge_schemas(v, from_field)
            to_schema.add_field(k, v)

    # Add remaining fields if we can
    can_add = lambda f: f[0] not in seen and _can_add_field(to_schema, f[0])
    for field in ifilter(can_add, from_schema.fields.iteritems()):
        to_schema.add_field(*field)

    return to_schema

class VariableDecoder(NestedVariables):
    pass

def _field_getter(children):
    return lambda name: children[name]



"""
Python-JS interface to dynamically create JS function calls from your widgets.

This moudle doesn't aim to serve as a Python-JS "translator". You should code
your client-side code in JavaScript and make it available in static files which
you include as JSLinks or inline using JSSources. This module is only intended
as a "bridge" or interface between Python and JavaScript so JS function 
**calls** can be generated programatically.
"""
import sys 

import logging
from itertools import imap
import simplejson.encoder

__all__ = ["js_callback", "js_function", "js_symbol", "encode"]

log = logging.getLogger(__name__)

class TWEncoder(simplejson.encoder.JSONEncoder):
    """A JSON encoder that can encode Widgets, js_calls, js_symbols and
    js_callbacks.
    
    Example::

        >>> encode = TWEncoder().encode
        >>> print encode({'onLoad': js_function("do_something")(js_symbol("this"))})
        {"onLoad": do_something(this)}

        >>> from tw.api import Widget
        >>> w = Widget("foo")
        >>> args = {'onLoad': js_callback(js_function('jQuery')(w).click(js_symbol('onClick')))}
        >>> print encode(args)
        {"onLoad": function(){jQuery(\\"foo\\").click(onClick)}}
        >>> print encode({'args':args})
        {"args": {"onLoad": function(){jQuery(\\"foo\\").click(onClick)}}}



    """
    def __init__(self, *args, **kw):
        self.pass_through = (_js_call, js_callback, js_symbol, js_function)
        super(TWEncoder, self).__init__(*args, **kw)

    def default(self, obj):
        if isinstance(obj, self.pass_through):
            return self.mark_for_escape(obj)
        elif hasattr(obj, '_id'):
            return str(obj.id)
        return super(TWEncoder, self).default(obj)

    def encode(self, obj):
        encoded = super(TWEncoder, self).encode(obj)
        return self.unescape_marked(encoded)

    def mark_for_escape(self, obj):
        return '*#*%s*#*' % obj

    def unescape_marked(self, encoded):
        return encoded.replace('"*#*','').replace('*#*"', '')
        

class js_symbol(object):
    def __init__(self, name):
        self._name = name
    def __str__(self):
        return str(self._name)

class js_callback(object):
    """A js function that can be passed as a callback to be called
    by another JS function

    Examples:

    .. sourcecode:: python

        >>> str(js_callback("update_div"))
        'update_div'

        >>> str(js_callback("function (event) { .... }"))
        'function (event) { .... }'

        # Can also create callbacks for deferred js calls

        >>> str(js_callback(js_function('foo')(1,2,3)))
        'function(){foo(1, 2, 3)}'

        # Or equivalently

        >>> str(js_callback(js_function('foo'), 1,2,3))
        'function(){foo(1, 2, 3)}'

        # A more realistic example
         
        >>> jQuery = js_function('jQuery')
        >>> my_cb = js_callback('function() { alert(this.text)}')
        >>> on_doc_load = jQuery('#foo').bind('click', my_cb)
        >>> call = jQuery(js_callback(on_doc_load))
        >>> print call
        jQuery(function(){jQuery(\\"#foo\\").bind(\\"click\\", function() { alert(this.text)})})

    """
    def __init__(self, cb, *args):
        if isinstance(cb, basestring):
            self.cb = cb
        elif isinstance(cb, js_function):
            self.cb = "function(){%s}" % cb(*args)
        elif isinstance(cb, _js_call):
            self.cb = "function(){%s}" % cb
        else:
            self.cb = ''

    def __call__(self, *args):
        raise TypeError("A js_callback cannot be called from Python")
    
    def __str__(self):
        return self.cb

class js_function(object):
    """A JS function that can be "called" from python and and added to
    a widget by widget.add_call() so it get's called every time the widget
    is rendered.

    Used to create a callable object that can be called from your widgets to
    trigger actions in the browser. It's used primarily to initialize JS code
    programatically. Calls can be chained and parameters are automatically
    json-encoded into something JavaScript undersrtands. Example::

    .. sourcecode:: python

        >>> jQuery = js_function('jQuery')
        >>> call = jQuery('#foo').datePicker({'option1': 'value1'})
        >>> str(call)
        'jQuery("#foo").datePicker({"option1": "value1"})'

    Calls are added to the widget call stack with the ``add_call`` method.

    If made at Widget initialization those calls will be placed in
    the template for every request that renders the widget.
        
    .. sourcecode:: python

        >>> from tw.api import Widget
        >>> class SomeWidget(Widget):
        ...     params = ["pickerOptions"]
        ...     pickerOptions = {}
        ...     def __init__(self, *args, **kw):
        ...         super(SomeWidget, self).__init__(*args, **kw)
        ...         self.add_call(
        ...             jQuery('#%s' % self.id).datePicker(self.pickerOptions)
        ...         )

    If we want to dynamically make calls on every request, we ca also add_calls
    inside the ``update_params`` method. 

    .. sourcecode:: python

        >>> class SomeWidget(Widget):
        ...     params = ["pickerOptions"]
        ...     pickerOptions = {}
        ...     def update_params(self, d):
        ...         super(SomeWidget, self).update_params(d)
        ...         self.add_call(
        ...             jQuery('#%s' % d.id).datePicker(d.pickerOptions)
        ...         )
    
    This would allow to pass different options to the datePicker on every
    display.

    JS calls are rendered by the same mechanisms that render required css and 
    js for a widget and places those calls at bodybottom so DOM elements which 
    we might target are available.

    Examples:

    .. sourcecode:: python

       >>> call = js_function('jQuery')("a .async")
       >>> str(call)
       'jQuery("a .async")'

       # js_function calls can be chained:

       >>> call = js_function('jQuery')("a .async").foo().bar()
       >>> str(call)
       'jQuery("a .async").foo().bar()'

    """
    def __init__(self, name):
        self.__name = name
            
    def __call__(self, *args):
        return _js_call(self.__name, [], args, called=True)
        

class _js_call(object):
    __slots__ = ('__name', '__call_list', '__args', '__called')
    
    def __init__(self, name, call_list, args=None, called=False):
        self.__name = name
        self.__args = args
        call_list.append(self)
        self.__call_list = call_list
        self.__called = called

    def __getattr__(self, name):
        return self.__class__(name, self.__call_list)
        
    def __call__(self, *args):
        self.__args = args
        self.__called = True
        return self
    
    def __get_js_repr(self):
        if self.__called:
            args = self.__args
            return '%s(%s)' % (self.__name, ', '.join(imap(encode, args)))
        else:
            return self.__name
    
    def __str__(self):
        if not self.__called:
            raise TypeError('Last element in the chain has to be called')
        return '.'.join(c.__get_js_repr() for c in self.__call_list)
    
    def __unicode__(self):
        return str(self).decode(sys.getdefaultencoding())

encode = TWEncoder().encode



import threading
import logging
import os
import errno
import string
import heapq
import mimetypes
from operator import itemgetter
from itertools import izip, chain, imap

from pkg_resources import resource_filename, working_set, Requirement, \
                          resource_stream
import tw
from tw.core.base import Widget
from tw.core.util import Enum, OrderedSet, RequestLocalDescriptor

__all__ = [
    "Resource", "Link", "JSLink", "CSSLink", "Source", "JSSource", "CSSSource",
    "locations", "merge_resources",
    "retrieve_resources", "JSFunctionCalls", "IECSSLink", "IECSSSource",
    "IEJSLink", "IEJSSource", "JSMixin", "CSSMixin"
    ]

log = logging.getLogger(__name__)


from webob import Request, Response

class ResourcesApp(object):
    def __init__(self, prefix='/resources', bufsize=4*1024):
        self._lock = threading.Lock()
        self._dirs = []
        self.prefix = prefix
        self.bufsize = bufsize

    def _is_registered(self, webdir, dirname):
        for old_wd, old_dn in self:
            if webdir == old_wd:
                if dirname != old_dn:
                    raise ValueError("%s is already regsitered for %s" %
                                     (webdir, old_dn))
                else:
                    return True
        return False

    def register(self, modname, filename):
        if isinstance(modname, Requirement):
            modname = os.path.basename(working_set.find(modname).location)

        basename = os.path.basename(filename)
        dirname = os.path.dirname(filename)
        parts = ['', modname] + filter(None, dirname.split('/'))
        webdir = '/'.join(parts)

        self._lock.acquire()
        try:
            if not self._is_registered(webdir, dirname):
                heapq.heappush(self._dirs, (len(webdir), (webdir, dirname)))
                log.debug("Regsitered %s at %s", dirname, webdir)
        finally:
            self._lock.release()
        url = '/'.join([self.prefix, webdir.strip('/'), basename])
        return webdir, dirname, url

    def get_prefixed(self):
        return tuple((self.prefix + k, v) for k,v in self)

    def __iter__(self):
        return imap(itemgetter(1), heapq.nlargest(len(self._dirs), self._dirs))

    def __call__(self, environ, start_response):
        req = Request(environ)
        path_info = req.path_info
        resp = self.serve_file(req)
        resp = resp or Response(status="404 Not Found")
        return resp(environ, start_response)

    def serve_file(self, req):
        stream, ct, enc = self.get_stream_type_encoding(req.path_info)
        if stream:
            resp = Response(request=req, app_iter=stream, content_type=ct)
            if enc:
                resp.content_type_params['charset'] = enc
            expires = int(req.environ.get('toscawidgets.resources_expire', 0))
            resp.cache_expires(expires)
            return resp
        else:
            return Response(status="404 Not Found")

    def is_resource(self, path_info):
        for webdir, dirname in self:
            if path_info.startswith(webdir):
                return True
        return False

    def get_stream_type_encoding(self, path_info):
        if not self.is_resource(path_info):
            return None, None, None
        parts = filter(None, path_info.split('/'))
        modname, relative_path = parts[0], '/'.join(parts[1:])
        try:
            stream = resource_stream(modname, relative_path)
            ct, enc  = mimetypes.guess_type(os.path.basename(relative_path))
            stream = _FileIter(stream, self.bufsize)
        except IOError, e:
            # For some reason pkg_resources sometimes sets errno to 0
            # when resource can't be found
            if e.errno != 0 and e.errno != errno.ENOENT:
                raise
            stream, ct, enc = None, None, None
        return stream, ct, enc


class _FileIter(object):
    def __init__(self, fileobj, bufsize):
        self.fileobj = fileobj
        self.bufsize = bufsize

    def __iter__(self):
        return self

    def next(self):
        buf = self.fileobj.read(self.bufsize)
        if not buf:
            raise StopIteration
        return buf

    def close(self):
        self.fileobj.close()


registry = ResourcesApp()

#------------------------------------------------------------------------------
# Base class for all resources
#------------------------------------------------------------------------------

class Resource(Widget):
    """
    A resource for your widget, like a link to external JS/CSS or inline
    source to include at the page the widget is displayed.

    It has the following parameters:

    `location`
        Location on the page where the resource should be placed. Available
        locations can be queried at ``Resource.valid_locations``
    """

    valid_locations = Enum('head','headbottom', 'bodytop', 'bodybottom')
    location = valid_locations.head

    def add_for_location(self, location):
        return location == self.location

    def inject(self):
        """
        Push this resource into request-local so it is injected to the page
        """
        tw.framework.register_resources(self.retrieve_resources())

    def register_resources(self):
        # A Resource is registered by other widgets, not by itself.
        pass

locations = Resource.valid_locations

#------------------------------------------------------------------------------
# Utility Mixins
#------------------------------------------------------------------------------

class CSSMixin:
    params = ["media"]
    media = "all"

    def post_init(self, *args, **kw):
        self._resources.add(self)


class JSMixin:
    def post_init(self, *args, **kw):
        self._resources.add(self)

class IEMixin:
    params = ["version"]
    version = ''

    _trans_table = ( ('>=',  '>',  '<=',  '<'), ('gte ', 'gt ', 'lte ', 'lt '))

    def _extend_template(self, d):
        d.version = str(d.version)
        for i, s in enumerate(self._trans_table[0]):
            d.version = d.version.replace(s, self._trans_table[1][i])
        s = ('', ' ')[int(bool(d.version))]
        d.template = "<!--[if IE%s${version}]>%s<![endif]-->"%(s,self.template)


#------------------------------------------------------------------------------
# Links
#------------------------------------------------------------------------------


class Link(Resource):
    """
    A link to an external resource, like a a JS or CSS file stored in the
    filesystem.

    Widgets automatically use the framework to register the directory where the
    resource is located so be careful that those directories contains no
    private data!
    """

    params = {
        'link': 'Use this to specify an external link. If provided this will '\
                'be used as-is as the resources URL. modname and filename '\
                'will be ignored.',
        'filename': "The relative path (from the module's or distribution's "\
                    "path) of the file the Link should link to.",
        'modname': "The module that contains the Widget declaration. "\
                   "If not given, it defaults to the name of the module where "\
                   "the Link is declared. Must be an existent module name. "\
                   "You can also pass a pkg_resources.Requirement instance to "\
                   "point to the root of an egg distribution."
        }
    _link = None
    filename = None
    modname = None

    def __init__(self, *args, **kw):
        super(Link, self).__init__(*args, **kw)
        if not self.is_external:
            modname = self.modname or self.__module__
            self.webdir, self.dirname, self.link = registry.register(
                modname, self.filename
                )
        else:
            self.link = kw.get('link')

    @property
    def is_external(self):
        return not bool(self.filename and self.modname)

    def _get_link(self):
        if self.is_external:
            return self._link
        return tw.framework.url(self._link or '')

    def _set_link(self, link):
        self._link = link

    link = property(_get_link, _set_link)

    def __hash__(self):
        return hash(self.link)

    def __eq__(self, other):
        return getattr(other, "link", None) == self.link



class CSSLink(Link,CSSMixin):
    """
    A link to an external CSS file.
    """
    template = """\
<link rel="stylesheet" type="text/css" href="$link" media="$media" />"""


class IECSSLink(CSSLink, IEMixin):
    def update_params(self, d):
        CSSLink.update_params(self, d)
        self._extend_template(d)


class JSLink(Link, JSMixin):
    template = """<script type="text/javascript" src="$link"></script>"""


class IEJSLink(JSLink, IEMixin):
    def update_params(self, d):
        JSLink.update_params(self, d)
        self._extend_template(d)

#------------------------------------------------------------------------------
# Raw source Resources
#------------------------------------------------------------------------------


class Source(Resource):
    """
    An inlined chunk of source code

    Examples::

        >>> class MySource(Source):
        ...     template = "$src"
        ...     src = "foo=$foo"
        ...     source_vars = ["foo"]
        ...     foo = "bar"
        ...
        >>> s = MySource()
        >>> s.render()
        u'foo=bar'

        >>> s = MySource(foo='zoo')
        >>> s.render()
        u'foo=zoo'

        >>> s.render(foo='foo')
        u'foo=foo'

        The whole source can also be overriden

        >>> s.render(src='foo')
        u'foo'
    """
    params = {
        'src': "A string with the source to include between the resource's "\
               "tags. Can also be a template for string.Template. Any "\
               "attribute listed at ``source_vars`` will be fetched from the "\
               "instance or from the kw args to ``display`` or ``render`` "\
               "into a dictionary to provide values to fill in."
        }


    def __new__(cls, *args, **kw):
        """Support positional params. (src)"""
        src = None
        parent = None
        if len(args) > 0:
            src = args[0]
        kw.setdefault('src', src or getattr(cls, 'src', None))
        if len(args) > 1:
            parent = args[1]
        return Resource.__new__(cls, None, parent, [], **kw)

    def update_params(self,d):
        super(Source, self).update_params(d)
        src = d.get('src')
        if src:
            source_vars = dict(
                v for v in [(k, getattr(self,k,None)) for k in self.source_vars]
                )
            source_vars.update(
                v for v in d.iteritems() if v[0] in self.source_vars
                )
            d['src'] = string.Template(src).safe_substitute(**source_vars)


    def __hash__(self):
        return hash(self.src)

    def __eq__(self, other):
        return self.src == getattr(other, "src", None)




class CSSSource(Source, CSSMixin):
    """
    An inlined chunk of CSS source code.
    """
    template = """<style type="text/css" media="$media">$src</style>"""


class IECSSSource(CSSSource, IEMixin):
    def update_params(self, d):
        super(IECSSSource, self).update_params(d)
        self._extend_template(d)


class JSSource(Source, JSMixin):
    """
    An inlined chunk of JS source code.
    """
    template = """<script type="text/javascript">$src</script>"""

class IEJSSource(JSSource, IEMixin):
    def update_params(self, d):
        JSSource.update_params(self, d)
        self._extend_template(d)



class JSFunctionCalls(JSSource):
    params = ["function_calls"]
    function_calls = []

    def __init__(self, id=None, parent=None, children=[], **kw):
        super(JSFunctionCalls, self).__init__(id, parent, children, **kw)
        self.src = "\n%s\n" % "\n".join(map(str, self.function_calls))



class JSDynamicFunctionCalls(JSFunctionCalls):
    params = ["call_getter"]
    call_getter = lambda s, location: []

    def update_params(self,d):
        super(JSDynamicFunctionCalls,self).update_params(d)
        # Keep in mind self._calls_for_request has calls for *all* widgets
        d.src = "\n%s\n" % "\n".join(
            map(str, chain(self.call_getter(self.location), d.function_calls))
            )

    # Since our src is generated dynamically base hash and equality on id
    def __hash__(self):
        return id(self)

    def __eq__(self, other):
        return id(self) == id(other)


# Utilities to retrieve resources


def merge_resources(to, from_):
    """
    In-place merge all resources from ``from_`` into ``to``. Resources
    from ``to_`` will come first in each resulting OrderedSet.
    """
    for k in locations:
        from_location = from_.get(k)
        if from_location:
            to.setdefault(k, OrderedSet()).add_all(from_location)
    return to

def retrieve_resources(obj):
    """Recursively retrieve resources from obj"""
    ret = {}
    if getattr(obj, 'retrieve_resources', None):
        ret = obj.retrieve_resources()
    elif getattr(obj, 'itervalues', None):
        ret = retrieve_resources(obj.itervalues())
    elif getattr(obj, '__iter__', None):
        ret = reduce(merge_resources, imap(retrieve_resources, iter(obj)), {})
    return ret

class DynamicCalls(object):
    _calls_for_request = RequestLocalDescriptor('_calls_for_request')
    def __init__(self):
        self.call_widgets = {}
        for l in locations:
            #XXX A circular-ref is probably created here with 'call_getter' but
            # since we're a module-level singleton we don't care much
            w = JSDynamicFunctionCalls('dynamic_js_calls_for_'+l, location=l,
                                       call_getter=self._get_calls_for_request)
            self.call_widgets[l] = w

    def reset(self):
        del self._calls_for_request

    def inject_call(self, call, location="bodybottom"):
        self._get_calls_for_request(location).append(call)
        self.call_widgets[location].inject()

    def _get_calls_for_request(self, location):
        try:
            self._calls_for_request
        except AttributeError:
            log.debug("Initializing calls for request")
            self._calls_for_request = dict((l,[]) for l in locations)
        return self._calls_for_request[location]

dynamic_js_calls = DynamicCalls()



import re
import warnings
import time
import hmac
from datetime import datetime, date
import weakref

import formencode
from formencode.validators import *
from formencode.foreach import ForEach
from formencode.compound import *
from formencode.api import Invalid, NoDefault
from formencode.schema import Schema

class DateTimeConverter(FancyValidator):

    """
    Converts Python date and datetime objects into string representation and back.
    """
    messages = {
        'badFormat': 'Invalid datetime format',
        'empty': 'Empty values not allowed',
    }
    if_missing = None
    def __init__(self, format = "%Y/%m/%d %H:%M", tzinfo=None, *args, **kwargs):
        super(FancyValidator, self).__init__(*args, **kwargs)
        self.format = format
        self.tzinfo = tzinfo

    def _to_python(self, value, state):
        """ parse a string and return a datetime object. """
        if value and isinstance(value, (date, datetime)):
            return value
        else:
            try:
                tpl = time.strptime(value, self.format)
            except ValueError:
                raise Invalid(self.message('badFormat', state), value, state)
            # shoudn't use time.mktime() because it can give OverflowError,
            # depending on the date (e.g. pre 1970) and underlying C library
            return datetime(year=tpl.tm_year, month=tpl.tm_mon, day=tpl.tm_mday,
                            hour=tpl.tm_hour, minute=tpl.tm_min,
                            second=tpl.tm_sec, tzinfo=self.tzinfo)

    def _from_python(self, value, state):
        if not value:
            return None
        elif isinstance(value, datetime):
            # Python stdlib can only handle dates with year greater than 1900
            if value.year <= 1900:
                return strftime_before1900(value, self.format)
            else:
                return value.strftime(self.format)
        else:
            return value

_illegal_s = re.compile(r"((^|[^%])(%%)*%s)")

def _findall(text, substr):
     # Also finds overlaps
     sites = []
     i = 0
     while 1:
         j = text.find(substr, i)
         if j == -1:
             break
         sites.append(j)
         i = j+1
     return sites

def strftime_before1900(dt, fmt):
    """
    A strftime implementation that supports proleptic Gregorian dates before 1900.

    @see: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/306860
    """
    import datetime
    if _illegal_s.search(fmt):
        raise TypeError("This strftime implementation does not handle %s")
    if dt.year > 1900:
        return dt.strftime(fmt)

    year = dt.year
    # For every non-leap year century, advance by
    # 6 years to get into the 28-year repeat cycle
    delta = 2000 - year
    off = 6*(delta // 100 + delta // 400)
    year = year + off

    # Move to around the year 2000
    year = year + ((2000 - year)//28)*28
    timetuple = dt.timetuple()
    s1 = time.strftime(fmt, (year,) + timetuple[1:])
    sites1 = _findall(s1, str(year))

    s2 = time.strftime(fmt, (year+28,) + timetuple[1:])
    sites2 = _findall(s2, str(year+28))

    sites = []
    for site in sites1:
        if site in sites2:
            sites.append(site)

    s = s1
    syear = "%4d" % (dt.year,)
    for site in sites:
        s = s[:site] + syear + s[site+4:]
    return s

class UnicodeString(UnicodeString):
    """The FormEncode UnicodeString validator encodes strings as utf-8 for display. However, this is not desired behaviour in tw.forms, as Genshi will fail when it receives such strings. Instead, this validator renders Python unicode objects where possible, strings otherwise."""
    def _from_python(self, value, state):
        if isinstance(value, basestring):
            return value
        elif hasattr(value, '__unicode__'):
            return unicode(value)
        else:
            return str(value)

class SecureTicketValidator(FancyValidator):
    """This validator helps you avoid cross-site request forgeries (CSRF) - a kind of web application security vulnerability."""
    def __init__(self, widget=None, session_secret_cb=None, **kw):
        # Weakref to container widget to avoid memory leaks
        widget = widget and weakref.proxy(widget) or widget
        self.widget = widget
        self.session_secret_cb = session_secret_cb

    def get_form_info(self):
        widgets = self.widget.root.walk()
        return '/'.join(["%s:%s" % (type(obj).__name__, obj.id) for obj in widgets])

    def get_hash(self):
        """
        Generate a hash that is associated with
            - Current user
            - Current session
            - The form
        """
        secret, stuff = map(str, self.session_secret_cb())
        stuff += self.get_form_info()
        return hmac.new(secret, stuff).hexdigest()

    def _to_python(self, value, state):
        if value != self.get_hash():
            msg = "Form token mismatch! Please try resubmitting the form."
            raise Invalid(msg, value, state)
        return None

__all__ = ["Invalid", "NoDefault"]
for name, value in locals().items():
    if isinstance(value, type) and issubclass(value, Validator):
        __all__.append(name)



import inspect
from pkg_resources import iter_entry_points

from tw.core.util import pre_post_hooks
from tw.core.exceptions import *


__all__ = [ "WidgetType", "WidgetsList"]

        
class WidgetType(type):
    def __new__(meta,name,bases,dct):
        dct['_cls_children'] = dct.pop('children',_children_from_bases(bases))
        if isinstance(dct.get('params'), dict):
            for param_name, doc in dct['params'].iteritems():
                dct[param_name+'__doc'] = doc
        params = frozenset_from_all_bases(dct,bases,'params')
        frozenset_from_all_bases(dct,bases,'source_vars')
        __init__ = dct.pop('__init__', None)
        if __init__:
            dct['__init__'] = pre_post_hooks(None, 'post_init')(__init__)
        new = type.__new__(meta,name,bases,dct)
        return new

    @classmethod
    def iter_classes(meta):
        """Iterates over all the Widget subclasses in the modules
        that are declared as `toscawidgets.widgets` entrypoints

        >>> from tw.api import Widget
        >>> len(list(Widget.iter_classes())) > 0
        True
        """
        seen = set()
        for ep in iter_entry_points('toscawidgets.widgets'):
            try:
                mod = ep.load(False)
            except ImportError, e:
                continue
            for name in dir(mod):
                obj = getattr(mod, name)
                if isinstance(obj, meta) and obj not in seen:
                    seen.add(obj)
                    yield obj

def _children_from_bases(bases):
    for base in bases:
        if hasattr(base, '_cls_children'):
            return base._cls_children
    return []

def frozenset_from_all_bases(clsdct, bases, name):
    _set = set(clsdct.pop(name,[]))
    [_set.update(getattr(b,name)) for b in bases if hasattr(b, name)]
    fs = clsdct[name] = frozenset(_set)
    return fs

    

class WidgetsListType(type):
    def __new__(meta,name,bases,dct):
        clones = []
        for id, w in dct.items():
            if hasattr(w,'_serial') and hasattr(w, 'clone'):
                dct.pop(id)
                clones.append((w._serial, id, w.clone) )
        clones.sort()
        def __init__(self, clones=clones): 
            # we instantiate the clones on initialization
            widgets = [w[2](id=w[1]) for w in clones]
            self.extend(widgets)
        dct.update({'__slots__':[], '__init__':__init__})
        return type.__new__(meta,name,bases,dct)
        
    def __add__(self, other):
        if isinstance(self, list) and isinstance(other,list):
            return list.__add__(self,other)

        elif isinstance(self, list) and inspect.isclass(other):
            return list.__add__(self,other())

        elif inspect.isclass(self) and isinstance(other,list):
            return list.__add__(self(),other)

        elif inspect.isclass(self) and inspect.isclass(other):
            return list.__add__(self(),other())


    def __radd__(self, other):
        if isinstance(self, list) and isinstance(other,list):
            return list.__add__(other,self)

        elif isinstance(self, list) and inspect.isclass(other):
            return list.__add__(other(),self)

        elif inspect.isclass(self) and isinstance(other,list):
            return list.__add__(other,self())

        elif inspect.isclass(self) and inspect.isclass(other):
            return list.__add__(other(),self())

class WidgetsList(list):
    """
    Syntactic sugar for declaring a list of widgets.

        >>> from tw.api import Widget
        >>> class Widgets(WidgetsList):
        ...     a = Widget()
        ...     b = Widget()
        ...     c = Widget()
        ...
        >>> widgets = Widgets()
        >>> [w.id for w in widgets]
        ['a', 'b', 'c']

    WidgetsLists can be passed to another widget as children by
    the instance or the class.

        >>> w = Widget('foo', children=widgets)
        >>> [c.id for c in w.children]
        ['foo_a', 'foo_b', 'foo_c']

        >>> w = Widget('foo', children=Widgets)
        >>> [c.id for c in w.children]
        ['foo_a', 'foo_b', 'foo_c']

    WidgetsLists subclasses can also be added to reuse common widgets

        >>> class Widgets2(WidgetsList):
        ...     d = Widget()
        ...
        >>> widgets = Widgets + Widgets2
        >>> [w.id for w in widgets]
        ['a', 'b', 'c', 'd']
        >>> widgets = Widgets2 + Widgets
        >>> [w.id for w in widgets]
        ['d', 'a', 'b', 'c']

    """
    __metaclass__ = WidgetsListType


if __name__ == "__main__":
    import doctest
    doctest.testmod()



class WidgetException(RuntimeError):
    msg = "Widget error"
    def __init__(self, msg=None):
        self.msg = msg or self.msg

    def __str__(self):
        return self.msg

class WidgetUnlocked(WidgetException, AttributeError):
    msg = ("The widget is not locked. This method needs to wait until the "
           "widget is fully locked in order to function properly")

class WidgetLocked(WidgetException, AttributeError):
    msg = ("The widget is locked. It's unthread-safe to alter it's attributes "
           "after initialization.")

class WidgetInitialized(WidgetException, AttributeError):
    msg = ("The widget is already initialized, try doing it at the "
           "constructor.")

class WidgetUninitialized(WidgetException, AttributeError):
    msg = ("The widget is uninitialized.")



"""Generic widget to present and manipulate data in a grid (tabular) form.

Adapted from turbogears.widgets.datagrid

"""

from tw.api import Widget, CSSLink

NoDefault = object()

__all__ = ["DataGrid"]


class attrwrapper(object):
    """Helper class that returns an object's attribute when called.

    This allows to access 'dynamic' attributes (properties) as well as
    simple static ones, and also allows nested access.

    """

    def __init__(self, name):
        assert isinstance(name, basestring)
        self.name = name

    def __call__(self, obj):
        for name in self.name.split('.'):
            obj = getattr(obj, name)
        return obj


class Column(object):
    """Simple struct that describes a single DataGrid column.

    Column has:
      - a name, which allows to uniquely identify a column in a DataGrid
      - a getter, which is used to extract the field's value
      - a title, which is displayed in the table's header
      - options, which is a way to carry arbitrary user-defined data

    """

    def __init__(self, name, getter=None, title=None, options=None):
        if not name:
            raise ValueError, 'name is required'
        if getter:
            if callable(getter):
                self.getter = getter
            else: # assume it's an attribute name
                self.getter = attrwrapper(getter)
        else:
            self.getter = attrwrapper(name)
        self.name = name
        self.title = title is None and name.capitalize() or title
        self.options = options or {}

    def get_option(self, name, default=NoDefault):
        if name in self.options:
            return self.options[name]
        if default is NoDefault: # no such key and no default is given
            raise KeyError(name)
        return default

    def get_field(self, row, displays_on=None):
        if isinstance(self.getter, Widget):
            return self.getter(row, displays_on=displays_on)
        return self.getter(row)

    def __str__(self):
        return "<Column %s>" % self.name


class DataGrid(Widget):
    """Generic widget to present and manipulate data in a grid (tabular) form.

    The columns to build the grid from are specified with fields constructor
    argument which is a list. An element can be a Column, an accessor
    (attribute name or function), a tuple (title, accessor) or a tuple
    (title, accessor, options).

    You can specify columns' data statically, via fields constructor parameter,
    or dynamically, via 'fields' key.

    """
    css_class = "grid"
    css=[CSSLink(modname='tw.forms', filename='static/grid.css')]
    template = "tw.forms.templates.datagrid"
    engine_name = "genshi"
    fields = []
    params = ["fields"]

    def get_column(self, name):
        """Return Column with specified name.

        Raises KeyError if no such column exists.

        """
        for col in self.columns:
            if col.name == name:
                return col
        raise KeyError(name)

    def __getitem__(self, name):
        """Shortcut to get_column."""
        return self.get_column(name)

    @staticmethod
    def get_field_getter(columns):
        """Return a function to access the fields of table by row, col."""
        idx = {} # index columns by name
        for col in columns:
            idx[col.name] = col
        def _get_field(row, col):
            return idx[col].get_field(row)
        return _get_field

    def update_params(self, d):
        super(DataGrid, self).update_params(d)
        if d.get('fields'):
            fields = d.pop('fields')
            columns = self._parse(fields)
        else:
            columns = self.columns[:]
        d['columns'] = columns
        d['get_field'] = self.get_field_getter(columns)
        # this is for backward compatibility
        d['headers'] = [col.title for col in columns]
        d['collist'] = [col.name for col in columns]

    def _parse(self, fields):
        """Parse field specifications into a list of Columns.

        A specification can be a Column,
        an accessor (attribute name or function), a tuple (title, accessor)
        or a tuple (title, accessor, options).

        """
        columns = []
        names = {} # keep track of names to ensure there are no dups
        for n, col in enumerate(fields):
            if not isinstance(col, Column):
                if isinstance(col, str) or callable(col):
                    name_or_f = col
                    title = options = None
                else:
                    title, name_or_f = col[:2]
                    try:
                        options = col[2]
                    except IndexError:
                        options = None
                # construct name using column index
                name = 'column-' + str(n)
                col = Column(name, name_or_f, title, options)
            if col.name in names:
                raise ValueError('Duplicate column name: %s' % col.name)
            columns.append(col)
            names[col.name] = 1
        return columns



"""Setuptools setup file"""

import sys, os

from setuptools import setup

if sys.version_info < (2, 4):
    raise SystemExit("Python 2.4 or later is required ATM")

execfile(os.path.join("tw", "release.py"))

def get_description(fname='README.txt'):
    # Adapted from PEAK-Rules' setup.py
    # Get our long description from the documentation
    f = file(fname)
    lines = []
    for line in f:
        if not line.strip():
            break     # skip to first blank line
    for line in f:
        if line.startswith('Documentation contents'):
            break     # read to "Documentation contents..."
        lines.append(line)
    f.close()
    return ''.join(lines)

PACKAGES = [
    'tw', 
    'tw.core',
    'tw.mods',
    ]

# Requirements to install buffet plugins and engines
_extra_cheetah = ["Cheetah>=1.0", "TurboCheetah>=0.9.5"]
_extra_genshi = ["Genshi >= 0.3.5"]
_extra_kid = ["kid>=0.9.5", "TurboKid>=0.9.9"]
_extra_mako = ["Mako >= 0.1.1"]

# Requierements to run all tests
_extra_tests = _extra_cheetah + _extra_genshi + _extra_kid + _extra_mako + ['BeautifulSoup', 'WebTest']

setup(
    name=__PACKAGE_NAME__,
    version=__VERSION__,
    description="Web widget creation toolkit based on TurboGears widgets",
    long_description = get_description(),
    install_requires=[
        'WebOb',
        'simplejson >= 2.0',
        ],
    extras_require = {
        'cheetah': _extra_cheetah,
        'kid': _extra_kid,
        'genshi': _extra_genshi,
        'mako': _extra_mako,
        'build_docs': [
            "Sphinx", 
            "WidgetBrowser", 
            ],
        },
    tests_require = _extra_tests,
    url = "http://toscawidgets.org/",
    download_url = "http://toscawidgets.org/download/",
    author=__AUTHOR__,
    author_email=__EMAIL__,
    license=__LICENSE__,
    test_suite = 'tests',
    packages = PACKAGES,
    namespace_packages = ['tw', 'tw.mods'],
    include_package_data=True,
    exclude_package_data={"thirdparty" : ["*"]},
    entry_points="""
    [distutils.commands]
    archive_tw_resources = tw.core.command:archive_tw_resources

    [python.templating.engines]
    toscawidgets = tw.core.engine_plugin:ToscaWidgetsTemplatePlugin

    [toscawidgets.host_frameworks]
    wsgi = tw.mods.wsgi:WSGIHostFramework
    pylons = tw.mods.pylonshf:PylonsHostFramework
    turbogears = tw.mods.tg:Turbogears

    [toscawidgets.widgets]
    widgets = tw.api
    resources = tw.api

    [paste.paster_create_template]
    toscawidgets=tw.paste_template:ToscaWidgetsTemplate

    [turbogears.extensions]
    toscawidgets=tw.mods.tg

    [paste.filter_app_factory]
    middleware = tw.api:make_middleware
    """,
    zip_safe=False,
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Environment :: Web Environment :: ToscaWidgets',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware',
        'Topic :: Software Development :: Widget Sets',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)



"""
Form widgets for ToscaWidgets.

To download and install::
   
   easy_install twForms
"""
from tw.api import Widget
from tw.forms.core import *
from tw.forms.fields import *
from tw.forms.datagrid import *
from tw.forms.calendars import *


# build all so doc tools introspect me properly
from tw.forms.core import __all__ as __core_all
from tw.forms.fields import __all__ as __fields_all
from tw.forms.datagrid import __all__ as __datagrid_all
from tw.forms.calendars import __all__ as __calendars_all
__all__ = __core_all + __fields_all + __datagrid_all + __calendars_all



from pkg_resources import require, VersionConflict

import os, logging

# TurboGears-1.0 uses CherryPy-2.3 while TurboGears-1.5 uses CherryPy-3.1. 
# CherryPy-2.3 uses Filters while CherryPy-3.1 uses Tools. We put the CherryPy
# specific code in separate files which makes it possible for CherryPy users 
# that are not using TurboGears to use them directly.
try:
    require("TurboGears>=1.0, <=1.5")
    import cp2 as cp
    default_view = 'kid'
except VersionConflict:
    require("TurboGears>=1.5a1dev, <=2.0")
    import cp3 as cp
    default_view = 'genshi'

import turbogears
from turbogears.i18n.tg_gettext import gettext
from turbogears.view import stdvars

import cherrypy

import tw
from tw.core import view
from tw.core.util import install_framework
from tw.mods.base import HostFramework

install_framework()

log = logging.getLogger(__name__)

class TurboGears(HostFramework):
    @property
    def request_local(self):
        try:
            rl = cherrypy.request.tw_request_local
        except AttributeError:
            rl = self.request_local_class(cherrypy.request.wsgi_environ)
            cherrypy.request.tw_request_local = rl
        return rl

    def start_request(self, environ):
        self.request_local.default_view = self._default_view
        
    def url(self, url):
        """
        Returns the absolute path for the given url.
        """
        prefix = self.request_local.environ['toscawidgets.prefix']
        
        return '/' + turbogears.url(prefix+url).lstrip('/')

def start_extension():
    if not cherrypy.config.get('toscawidgets.on', False):
        return

    engines = view.EngineManager()
    engines.load_all(cp._extract_config(), stdvars)
    
    host_framework = TurboGears(
        engines = engines,
        default_view = cherrypy.config.get('tg.defaultview', default_view),
        translator = gettext,
        )
    prefix = cherrypy.config.get('toscawidgets.prefix', '/toscawidgets')
    host_framework.prefix = prefix
    host_framework.webpath = cherrypy.config.get('server.webpath', '')
    
    log.info("Loaded TW TurboGears HostFramework")
    filter_args = dict(
        prefix = prefix,
        serve_files = cherrypy.config.get('toscawidgets.serve_files', 1)
        )
    
    cp.start_extension(host_framework, **filter_args)



from datetime import datetime
import logging
import errno

import tw
from tw.api import CSSLink, JSLink, js_function
from tw.forms import FormField, validators


__all__ = ["CalendarDatePicker", "CalendarDateTimePicker"]

setup_calendar = js_function("Calendar.setup")

log = logging.getLogger(__name__)

calendar_css = CSSLink(
    modname='tw.forms', filename='static/calendar/calendar-system.css')
calendar_js = JSLink(
    modname='tw.forms', filename='static/calendar/calendar.js')
calendar_setup = JSLink(
    modname='tw.forms', filename='static/calendar/calendar-setup.js')

class CalendarDatePicker(FormField):
    """
    Uses a javascript calendar system to allow picking of calendar dates.
    The date_format is in mm/dd/yyyy unless otherwise specified
    """
    css = [calendar_css]
    javascript = [calendar_js, calendar_setup]
    template = "tw.forms.templates.calendar"
    params = [
        "calendar_lang", "not_empty", "button_text", "date_format",
        "picker_shows_time", "tzinfo",
        ]
    calendar_lang = 'en'
    not_empty = True
    button_text = "Choose"
    date_format = "%m/%d/%Y"
    picker_shows_time = False
    validator = None

    _default = None

    def __init__(self, *args, **kw):
        super(CalendarDatePicker, self).__init__(*args, **kw)
        if self.default is None and self.not_empty:
            self.default = lambda: datetime.now()
        self.validator = self.validator or validators.DateTimeConverter(
            format=self.date_format, not_empty=self.not_empty,
            tzinfo=self.tzinfo
            )


    def get_calendar_lang_file_link(self, lang):
        """
        Returns a CalendarLangFileLink containing a list of name
        patterns to try in turn to find the correct calendar locale
        file to use.
        """
        fname = 'static/calendar/lang/calendar-%s.js' % lang.lower()
        return JSLink(modname='tw.forms',
                      filename=fname,
                      javascript=self.javascript)

    def update_params(self, d):
        super(CalendarDatePicker, self).update_params(d)
        log.debug("Value received by Calendar: %r", d.value)
        try:
            d.strdate = d.value.strftime(d.date_format)
        except AttributeError:
            d.strdate = d.value
        options = dict(
            inputField = self.id,
            ifFormat = d.date_format,
            button = self.id + '_trigger',
            showsTime = d.picker_shows_time,
            )
        self.get_calendar_lang_file_link(d.calendar_lang).inject()
        self.add_call(setup_calendar(options))


class CalendarDateTimePicker(CalendarDatePicker):
    """
    Use a javascript calendar system to allow picking of calendar dates and
    time.
    The date_format is in mm/dd/yyyy hh:mm unless otherwise specified
    """
    date_format = "%Y/%m/%d %H:%M"
    picker_shows_time = True



from urllib import unquote

import pkg_resources

from webob import Request

import tw
from tw.mods import base
from tw.core import resources
from tw.core.util import asbool

__all__ = ["ToscaWidgetsMiddleware", "make_middleware"]
    
class ToscaWidgetsMiddleware(object):
    """
    This WSGI middleware piece takes care of creating a per-request context for
    ToscaWidgets and injecting resource links into html responses.

    It can also take care of serving those resources if `serve_resources` is
    True (default).
    """
    def __init__(self, application, host_framework, prefix='/toscawidgets',
                 inject_resources=True, serve_resources=True):
        self.host_framework = host_framework
        self.prefix = prefix
        self.serve_resources = serve_resources
        self.inject_resources = inject_resources

        self.application = application

        if self.inject_resources:
            from tw.core.resource_injector import injector_middleware
            self.application = injector_middleware(self.application)

    def __call__(self, environ, start_response):
        return self.wsgi_app(environ, start_response)

    def wsgi_app(self, environ, start_response):
        self.host_framework.start_request(environ)
        environ['paste.registry'].register(tw.framework, self.host_framework)
        #XXX Do we really need to stuff these in environ?
        environ['toscawidgets.prefix'] = self.prefix
        environ.setdefault('toscawidgets.framework', self.host_framework)

        req = Request(environ)
        try:
            if self.serve_resources and req.path_info.startswith(self.prefix):
                # Intercept request to possibly serve a static resource
                req.path_info = req.path_info[len(self.prefix):]
                req.script_name += self.prefix
                resources_app = resources.registry
                if req.path_info.startswith(resources_app.prefix):
                    req.path_info = req.path_info[len(resources_app.prefix):]
                    req.script_name += resources_app.prefix
                    resp = req.get_response(resources_app)
                    return resp(environ, start_response)
            else:
                # Pass request downstream  
                resp = req.get_response(self.application)
            return resp(environ, start_response)
        finally:
            self.host_framework.end_request(environ)

#TODO: Wrap to issue deprecation warnings
TGWidgetsMiddleware = ToscaWidgetsMiddleware

def _load_from_entry_point(name):
    for ep in pkg_resources.iter_entry_points('toscawidgets.host_frameworks'):
        if ep.name == name:
            return ep
        
def _extract_args(args, prefix, adapters={}):
    l = len(prefix)
    nop = lambda v: v
    return dict((k[l:], adapters.get(k[l:], nop)(v))
                for k,v in args.iteritems() if k.startswith(prefix))
    
def _load_host_framework(host_framework):
    if ':' not in host_framework:
        ep = _load_from_entry_point(host_framework)
    else:
        ep = pkg_resources.EntryPoint.parse("hf="+host_framework)
    if ep:
        hf = ep.load(False)
    else:
        hf = None
    if not hf:
        raise LookupError("Could not load %s" % host_framework)
    return hf

def make_middleware(app, config=None, **kw):
    """
    Initializes :class:`tw.core.middleware.ToscaWidgetsMiddleware` and a
    :class:`tw.mods.base.HostFramework` and wraps the WSGI application ``app``
    with it.

    Configuration can be passed in a dict as the ``config`` parameter.


    **Available options:**

    toscawidgets.middleware.*
        These parameters will be passed to
        :class:`tw.core.middleware.ToscaWidgetsMiddleware`
        when instatntiating it. See its docstrings for details.

    toscawidgets.framework
        Name of the ``toscawidgets.host_frameworks`` entry point or
        :class:`tw.mods.base.HostFramework` subclass which shall interface with
        the framework. ``wsgi``, ``pylons`` are available. Default is
        ``wsgi``

    toscawidgets.framework.*
        Parameters for the :class:`tw.modes.base.HostFramework`. See their
        respective docstrings for accepted parameters.


    This is the ToscaWidgets#middleware paste.filter_app_factory entrypoint.
    """
    config = (config or {}).copy()
    config.update(kw)
    host_framework = config.pop('toscawidgets.framework', 'wsgi')
    if isinstance(host_framework, basestring):
        host_framework = _load_host_framework(host_framework)
    middleware_args = _extract_args(config, 'toscawidgets.middleware.', {
        'inject_resources': asbool,
        'serve_resources': asbool,
        })
    hf_args = _extract_args(config, 'toscawidgets.framework.', {
        'enable_runtime_checks': asbool,
        })
    app = ToscaWidgetsMiddleware(app, host_framework=host_framework(**hf_args),
                                 **middleware_args)
    if config.get('stack_registry', False):
        from tw.core.registry import RegistryManager
        app = RegistryManager(app)
    return app



#!/usr/bin/env python2.4

"""Setuptools setup file"""

import sys, os
from setuptools import setup, find_packages

if sys.version_info < (2, 4):
    raise SystemExit("Python 2.4 or later is required ATM")

execfile(os.path.join("tw", "forms", "release.py"))

setup(
    name=__PACKAGE_NAME__,
    version=__VERSION__,
    description="Web Widgets for building and validating forms. (former ToscaWidgetsForms)",
    #long_description = "",
    install_requires=[
        'ToscaWidgets >= 0.9.7.2',
        'FormEncode >= 1.1',
        ],
    extras_require = dict(
        mako = ['Mako'],
        genshi = ['Genshi >= 0.3.6'],
        ),
    url = "http://toscawidgets.org",
    download_url = "http://toscawidgets.org/download",
    author=__AUTHOR__,
    author_email=__EMAIL__,
    license=__LICENSE__,
    test_suite = 'tests',
    packages=find_packages(exclude=['ez_setup', 'tests']),
    namespace_packages = ['tw'],
    include_package_data=True,
    zip_safe=False,
    entry_points="""
    [toscawidgets.widgets]
    widgets = tw.forms
    samples = tw.forms.samples
    """,
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Environment :: Web Environment :: ToscaWidgets',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Widget Sets',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        ],
)
